"""Hardware Monitoring client implementation."""
from typing import Dict, Any, Optional, List, Union, Callable
import threading
import time
import datetime
import os
import json

from greenfish.utils.logger import Logger
from greenfish.core.hwmonitor.types import SensorReading, SensorType, HealthStatus
from greenfish.core.hwmonitor.alert import AlertManager, Alert, AlertRule, AlertLevel

class HardwareMonitorClient:
    """Hardware Monitor client for monitoring BMC hardware."""

    def __init__(self, config_dir: str = None):
        """Initialize Hardware Monitor client.

        Args:
            config_dir: Configuration directory
        """
        self.redfish_client = None
        self.ipmi_client = None
        self.connection_type = None
        self.connected = False
        self.sensor_cache = {}
        self.health_status = {}
        self.alert_manager = AlertManager(config_dir)
        self.polling_thread = None
        self.polling_interval = 60  # seconds
        self.polling_enabled = False
        self.sensor_callbacks = []
        self.health_callbacks = []

    def connect_redfish(self, redfish_client) -> bool:
        """Connect using Redfish client.

        Args:
            redfish_client: Redfish client

        Returns:
            bool: True if successful
        """
        try:
            if not redfish_client or not redfish_client.is_connected():
                Logger.error("Redfish client is not connected")
                return False

            self.redfish_client = redfish_client
            self.connection_type = "redfish"
            self.connected = True

            # Clear cache
            self.sensor_cache = {}
            self.health_status = {}

            return True

        except Exception as e:
            Logger.error(f"Error connecting Hardware Monitor client: {str(e)}")
            self.connected = False
            return False

    def connect_ipmi(self, ipmi_client) -> bool:
        """Connect using IPMI client.

        Args:
            ipmi_client: IPMI client

        Returns:
            bool: True if successful
        """
        try:
            if not ipmi_client or not ipmi_client.is_connected():
                Logger.error("IPMI client is not connected")
                return False

            self.ipmi_client = ipmi_client
            self.connection_type = "ipmi"
            self.connected = True

            # Clear cache
            self.sensor_cache = {}
            self.health_status = {}

            return True

        except Exception as e:
            Logger.error(f"Error connecting Hardware Monitor client: {str(e)}")
            self.connected = False
            return False

    def disconnect(self) -> bool:
        """Disconnect client.

        Returns:
            bool: True if successful
        """
        # Stop polling
        self.stop_polling()

        self.redfish_client = None
        self.ipmi_client = None
        self.connection_type = None
        self.connected = False
        self.sensor_cache = {}
        self.health_status = {}
        return True

    def is_connected(self) -> bool:
        """Check if client is connected.

        Returns:
            bool: True if connected
        """
        return self.connected

    def _get_redfish_sensors(self) -> List[SensorReading]:
        """Get sensors from Redfish.

        Returns:
            List[SensorReading]: List of sensor readings
        """
        sensors = []

        try:
            if not self.redfish_client or not self.connected:
                Logger.error("Not connected to Redfish")
                return []

            # Get thermal sensors
            thermal_data = self.redfish_client.get_thermal()
            if thermal_data:
                # Process temperature sensors
                for sensor in thermal_data.get("Temperatures", []):
                    sensor_reading = SensorReading.from_redfish(sensor)
                    sensors.append(sensor_reading)

                # Process fans
                for sensor in thermal_data.get("Fans", []):
                    sensor_reading = SensorReading.from_redfish(sensor)
                    sensors.append(sensor_reading)

            # Get power sensors
            power_data = self.redfish_client.get_power()
            if power_data:
                # Process power supplies
                for sensor in power_data.get("PowerSupplies", []):
                    # Create sensor for power supply status
                    status_sensor = {
                        "Id": f"{sensor.get('Id', '')}_Status",
                        "Name": f"{sensor.get('Name', '')} Status",
                        "Status": sensor.get("Status", {}),
                        "SensorType": "HealthState"
                    }
                    sensor_reading = SensorReading.from_redfish(status_sensor)
                    sensors.append(sensor_reading)

                    # Create sensor for power output
                    if "PowerOutputWatts" in sensor:
                        output_sensor = {
                            "Id": f"{sensor.get('Id', '')}_Output",
                            "Name": f"{sensor.get('Name', '')} Output",
                            "Reading": sensor.get("PowerOutputWatts"),
                            "ReadingUnits": "Watts",
                            "SensorType": "Power"
                        }
                        sensor_reading = SensorReading.from_redfish(output_sensor)
                        sensors.append(sensor_reading)

                    # Create sensor for input voltage
                    if "LineInputVoltage" in sensor:
                        voltage_sensor = {
                            "Id": f"{sensor.get('Id', '')}_Voltage",
                            "Name": f"{sensor.get('Name', '')} Input Voltage",
                            "Reading": sensor.get("LineInputVoltage"),
                            "ReadingUnits": "Volts",
                            "SensorType": "Voltage"
                        }
                        sensor_reading = SensorReading.from_redfish(voltage_sensor)
                        sensors.append(sensor_reading)

                # Process voltage sensors
                for sensor in power_data.get("Voltages", []):
                    sensor_reading = SensorReading.from_redfish(sensor)
                    sensors.append(sensor_reading)

            # Get additional sensors from Chassis
            chassis_list = self.redfish_client.get_chassis()
            for chassis in chassis_list:
                chassis_id = chassis.get("Id", "")

                # Get sensors collection
                sensors_uri = chassis.get("Sensors", {}).get("@odata.id")
                if sensors_uri:
                    sensors_data = self.redfish_client.get(sensors_uri)
                    if sensors_data and "Members" in sensors_data:
                        for member in sensors_data["Members"]:
                            sensor_uri = member.get("@odata.id")
                            if sensor_uri:
                                sensor_data = self.redfish_client.get(sensor_uri)
                                if sensor_data:
                                    sensor_reading = SensorReading.from_redfish(sensor_data)
                                    sensors.append(sensor_reading)

            return sensors

        except Exception as e:
            Logger.error(f"Error getting Redfish sensors: {str(e)}")
            return []

    def _get_ipmi_sensors(self) -> List[SensorReading]:
        """Get sensors from IPMI.

        Returns:
            List[SensorReading]: List of sensor readings
        """
        sensors = []

        try:
            if not self.ipmi_client or not self.connected:
                Logger.error("Not connected to IPMI")
                return []

            # Get sensor readings
            sensor_data = self.ipmi_client.get_sensor_readings()

            # Convert to SensorReading objects
            for sensor in sensor_data:
                sensor_reading = SensorReading.from_ipmi(sensor)
                sensors.append(sensor_reading)

            return sensors

        except Exception as e:
            Logger.error(f"Error getting IPMI sensors: {str(e)}")
            return []

    def get_sensors(self, sensor_type: Optional[SensorType] = None, use_cache: bool = False) -> List[SensorReading]:
        """Get sensor readings.

        Args:
            sensor_type: Filter by sensor type
            use_cache: Use cached sensor readings

        Returns:
            List[SensorReading]: List of sensor readings
        """
        try:
            if not self.connected:
                Logger.error("Not connected")
                return []

            # Check if using cache
            if use_cache and self.sensor_cache:
                sensors = list(self.sensor_cache.values())
            else:
                # Get sensors based on connection type
                if self.connection_type == "redfish":
                    sensors = self._get_redfish_sensors()
                elif self.connection_type == "ipmi":
                    sensors = self._get_ipmi_sensors()
                else:
                    sensors = []

                # Update cache
                for sensor in sensors:
                    self.sensor_cache[sensor.id] = sensor

                # Process alerts
                self._process_alerts(sensors)

                # Notify callbacks
                for callback in self.sensor_callbacks:
                    try:
                        callback(sensors)
                    except Exception as e:
                        Logger.error(f"Error in sensor callback: {str(e)}")

            # Filter by sensor type if specified
            if sensor_type:
                sensors = [s for s in sensors if s.sensor_type == sensor_type]

            return sensors

        except Exception as e:
            Logger.error(f"Error getting sensors: {str(e)}")
            return []

    def get_sensor(self, sensor_id: str, use_cache: bool = False) -> Optional[SensorReading]:
        """Get sensor reading by ID.

        Args:
            sensor_id: Sensor ID
            use_cache: Use cached sensor reading

        Returns:
            SensorReading: Sensor reading
        """
        try:
            if not self.connected:
                Logger.error("Not connected")
                return None

            # Check if using cache
            if use_cache and sensor_id in self.sensor_cache:
                return self.sensor_cache[sensor_id]

            # Get all sensors
            sensors = self.get_sensors(use_cache=False)

            # Find sensor with matching ID
            for sensor in sensors:
                if sensor.id == sensor_id:
                    return sensor

            return None

        except Exception as e:
            Logger.error(f"Error getting sensor: {str(e)}")
            return None

    def _get_redfish_health(self) -> Dict[str, HealthStatus]:
        """Get health status from Redfish.

        Returns:
            Dict[str, HealthStatus]: Health status by component
        """
        health_status = {}

        try:
            if not self.redfish_client or not self.connected:
                Logger.error("Not connected to Redfish")
                return {}

            # Get system health
            systems = self.redfish_client.get_systems()
            for system in systems:
                system_id = system.get("Id", "")
                status = system.get("Status", {})
                health = status.get("Health", "")
                health_status[f"System_{system_id}"] = HealthStatus.from_string(health)

                # Get processor health
                processors_uri = system.get("Processors", {}).get("@odata.id")
                if processors_uri:
                    processors = self.redfish_client.get(processors_uri)
                    if processors and "Members" in processors:
                        for i, member in enumerate(processors["Members"]):
                            processor_uri = member.get("@odata.id")
                            if processor_uri:
                                processor = self.redfish_client.get(processor_uri)
                                if processor:
                                    processor_id = processor.get("Id", f"CPU{i}")
                                    status = processor.get("Status", {})
                                    health = status.get("Health", "")
                                    health_status[f"Processor_{processor_id}"] = HealthStatus.from_string(health)

                # Get memory health
                memory_uri = system.get("Memory", {}).get("@odata.id")
                if memory_uri:
                    memory = self.redfish_client.get(memory_uri)
                    if memory and "Members" in memory:
                        for i, member in enumerate(memory["Members"]):
                            dimm_uri = member.get("@odata.id")
                            if dimm_uri:
                                dimm = self.redfish_client.get(dimm_uri)
                                if dimm:
                                    dimm_id = dimm.get("Id", f"DIMM{i}")
                                    status = dimm.get("Status", {})
                                    health = status.get("Health", "")
                                    health_status[f"Memory_{dimm_id}"] = HealthStatus.from_string(health)

            # Get chassis health
            chassis_list = self.redfish_client.get_chassis()
            for chassis in chassis_list:
                chassis_id = chassis.get("Id", "")
                status = chassis.get("Status", {})
                health = status.get("Health", "")
                health_status[f"Chassis_{chassis_id}"] = HealthStatus.from_string(health)

            # Get manager health
            managers = self.redfish_client.get_managers()
            for manager in managers:
                manager_id = manager.get("Id", "")
                status = manager.get("Status", {})
                health = status.get("Health", "")
                health_status[f"Manager_{manager_id}"] = HealthStatus.from_string(health)

            return health_status

        except Exception as e:
            Logger.error(f"Error getting Redfish health: {str(e)}")
            return {}

    def _get_ipmi_health(self) -> Dict[str, HealthStatus]:
        """Get health status from IPMI.

        Returns:
            Dict[str, HealthStatus]: Health status by component
        """
        health_status = {}

        try:
            if not self.ipmi_client or not self.connected:
                Logger.error("Not connected to IPMI")
                return {}

            # Get system health from SEL
            sel_entries = self.ipmi_client.sel.get_sel_info()
            if sel_entries:
                # Check if there are any critical SEL entries
                critical_entries = False
                warning_entries = False

                # Get recent SEL entries
                entries = self.ipmi_client.sel.get_sel_entries(count=10)

                for entry in entries:
                    severity = entry.get("severity", "").lower()
                    if severity in ["critical", "error", "fatal"]:
                        critical_entries = True
                        break
                    elif severity in ["warning", "non-critical"]:
                        warning_entries = True

                if critical_entries:
                    health_status["System"] = HealthStatus.CRITICAL
                elif warning_entries:
                    health_status["System"] = HealthStatus.WARNING
                else:
                    health_status["System"] = HealthStatus.OK

            # Get health from sensor readings
            sensors = self._get_ipmi_sensors()

            # Group sensors by type
            sensor_groups = {}
            for sensor in sensors:
                sensor_type = sensor.sensor_type.value
                if sensor_type not in sensor_groups:
                    sensor_groups[sensor_type] = []
                sensor_groups[sensor_type].append(sensor)

            # Determine health status for each sensor group
            for sensor_type, group_sensors in sensor_groups.items():
                critical_count = 0
                warning_count = 0
                ok_count = 0

                for sensor in group_sensors:
                    if sensor.status == HealthStatus.CRITICAL:
                        critical_count += 1
                    elif sensor.status == HealthStatus.WARNING:
                        warning_count += 1
                    elif sensor.status == HealthStatus.OK:
                        ok_count += 1

                if critical_count > 0:
                    health_status[sensor_type] = HealthStatus.CRITICAL
                elif warning_count > 0:
                    health_status[sensor_type] = HealthStatus.WARNING
                elif ok_count > 0:
                    health_status[sensor_type] = HealthStatus.OK
                else:
                    health_status[sensor_type] = HealthStatus.UNKNOWN

            return health_status

        except Exception as e:
            Logger.error(f"Error getting IPMI health: {str(e)}")
            return {}

    def get_health(self, use_cache: bool = False) -> Dict[str, HealthStatus]:
        """Get health status.

        Args:
            use_cache: Use cached health status

        Returns:
            Dict[str, HealthStatus]: Health status by component
        """
        try:
            if not self.connected:
                Logger.error("Not connected")
                return {}

            # Check if using cache
            if use_cache and self.health_status:
                return self.health_status

            # Get health based on connection type
            if self.connection_type == "redfish":
                health_status = self._get_redfish_health()
            elif self.connection_type == "ipmi":
                health_status = self._get_ipmi_health()
            else:
                health_status = {}

            # Determine overall health
            if health_status:
                critical_count = 0
                warning_count = 0
                ok_count = 0

                for component, status in health_status.items():
                    if status == HealthStatus.CRITICAL:
                        critical_count += 1
                    elif status == HealthStatus.WARNING:
                        warning_count += 1
                    elif status == HealthStatus.OK:
                        ok_count += 1

                if critical_count > 0:
                    health_status["Overall"] = HealthStatus.CRITICAL
                elif warning_count > 0:
                    health_status["Overall"] = HealthStatus.WARNING
                elif ok_count > 0:
                    health_status["Overall"] = HealthStatus.OK
                else:
                    health_status["Overall"] = HealthStatus.UNKNOWN

            # Update cache
            self.health_status = health_status

            # Notify callbacks
            for callback in self.health_callbacks:
                try:
                    callback(health_status)
                except Exception as e:
                    Logger.error(f"Error in health callback: {str(e)}")

            return health_status

        except Exception as e:
            Logger.error(f"Error getting health status: {str(e)}")
            return {}

    def _process_alerts(self, sensors: List[SensorReading]) -> List[Alert]:
        """Process sensor readings and generate alerts.

        Args:
            sensors: List of sensor readings

        Returns:
            List[Alert]: Generated alerts
        """
        alerts = []

        try:
            for sensor in sensors:
                # Process sensor with alert manager
                sensor_alerts = self.alert_manager.process_sensor_reading(sensor)
                alerts.extend(sensor_alerts)

            return alerts

        except Exception as e:
            Logger.error(f"Error processing alerts: {str(e)}")
            return []

    def start_polling(self, interval: int = 60) -> bool:
        """Start polling for sensor readings.

        Args:
            interval: Polling interval in seconds

        Returns:
            bool: True if successful
        """
        try:
            if not self.connected:
                Logger.error("Not connected")
                return False

            # Stop existing polling
            self.stop_polling()

            # Set polling interval
            self.polling_interval = interval
            self.polling_enabled = True

            # Start polling thread
            self.polling_thread = threading.Thread(target=self._polling_loop)
            self.polling_thread.daemon = True
            self.polling_thread.start()

            Logger.info(f"Started hardware monitoring polling (interval: {interval}s)")
            return True

        except Exception as e:
            Logger.error(f"Error starting polling: {str(e)}")
            return False

    def stop_polling(self) -> bool:
        """Stop polling for sensor readings.

        Returns:
            bool: True if successful
        """
        try:
            # Stop polling
            self.polling_enabled = False

            # Wait for thread to terminate
            if self.polling_thread and self.polling_thread.is_alive():
                self.polling_thread.join(timeout=2)

            self.polling_thread = None

            Logger.info("Stopped hardware monitoring polling")
            return True

        except Exception as e:
            Logger.error(f"Error stopping polling: {str(e)}")
            return False

    def _polling_loop(self) -> None:
        """Polling loop for sensor readings."""
        try:
            while self.polling_enabled and self.connected:
                # Get sensor readings
                self.get_sensors(use_cache=False)

                # Get health status
                self.get_health(use_cache=False)

                # Sleep for polling interval
                time.sleep(self.polling_interval)

        except Exception as e:
            Logger.error(f"Error in polling loop: {str(e)}")
            self.polling_enabled = False

    def register_sensor_callback(self, callback: Callable[[List[SensorReading]], None]) -> None:
        """Register callback for sensor updates.

        Args:
            callback: Callback function
        """
        self.sensor_callbacks.append(callback)

    def register_health_callback(self, callback: Callable[[Dict[str, HealthStatus]], None]) -> None:
        """Register callback for health status updates.

        Args:
            callback: Callback function
        """
        self.health_callbacks.append(callback)

    def get_alerts(self,
                  max_count: int = 100,
                  level: Optional[AlertLevel] = None,
                  acknowledged: Optional[bool] = None,
                  sensor_type: Optional[SensorType] = None) -> List[Alert]:
        """Get alerts with optional filtering.

        Args:
            max_count: Maximum number of alerts to return
            level: Filter by alert level
            acknowledged: Filter by acknowledged status
            sensor_type: Filter by sensor type

        Returns:
            List[Alert]: List of alerts
        """
        return self.alert_manager.get_alerts(max_count, level, acknowledged, sensor_type)

    def acknowledge_alert(self, alert_id: str, user: str = None) -> bool:
        """Acknowledge alert.

        Args:
            alert_id: Alert ID
            user: User who acknowledged

        Returns:
            bool: True if successful
        """
        return self.alert_manager.acknowledge_alert(alert_id, user)

    def clear_alerts(self, acknowledged_only: bool = False) -> int:
        """Clear alerts.

        Args:
            acknowledged_only: Clear only acknowledged alerts

        Returns:
            int: Number of alerts cleared
        """
        return self.alert_manager.clear_alerts(acknowledged_only)

    def get_alert_rules(self) -> List[AlertRule]:
        """Get alert rules.

        Returns:
            List[AlertRule]: List of alert rules
        """
        return self.alert_manager.get_rules()

    def add_alert_rule(self, rule: AlertRule) -> bool:
        """Add alert rule.

        Args:
            rule: Alert rule

        Returns:
            bool: True if successful
        """
        return self.alert_manager.add_rule(rule)

    def remove_alert_rule(self, rule_id: str) -> bool:
        """Remove alert rule.

        Args:
            rule_id: Rule ID

        Returns:
            bool: True if successful
        """
        return self.alert_manager.remove_rule(rule_id)

    def register_alert_callback(self, callback: Callable[[Alert], None]) -> None:
        """Register callback for new alerts.

        Args:
            callback: Callback function
        """
        self.alert_manager.register_alert_callback(callback)



"""iPXE client implementation for bare metal provisioning."""
from typing import Dict, Any, Optional, List, Union
import os
import subprocess
import tempfile
import shutil
import re
import ftplib
import http.server
import socketserver
import threading
import time
from pathlib import Path

from greenfish.utils.logger import Logger

class IPXEClient:
    """iPXE client for bare metal provisioning."""

    def __init__(self):
        """Initialize iPXE client."""
        self.server_thread = None
        self.http_server = None
        self.tftp_server = None
        self.http_port = 8000
        self.tftp_port = 69
        self.ipxe_root = None
        self.running = False

    def setup_ipxe_environment(self, ipxe_root: str) -> bool:
        """Set up iPXE environment.

        Args:
            ipxe_root: Root directory for iPXE files

        Returns:
            bool: True if setup successful
        """
        try:
            self.ipxe_root = Path(ipxe_root).resolve()

            # Create directory structure if it doesn't exist
            os.makedirs(self.ipxe_root / "boot", exist_ok=True)
            os.makedirs(self.ipxe_root / "images", exist_ok=True)
            os.makedirs(self.ipxe_root / "scripts", exist_ok=True)

            # Create default iPXE script
            self._create_default_script()

            Logger.info(f"iPXE environment set up at {self.ipxe_root}")
            return True

        except Exception as e:
            Logger.error(f"Failed to set up iPXE environment: {str(e)}")
            return False

    def _create_default_script(self) -> None:
        """Create default iPXE script."""
        default_script = """#!ipxe

echo ========================================
echo Redfish Desktop iPXE Boot Menu
echo ========================================
echo

:menu
menu iPXE Boot Menu
item --gap -- ------------------------- OS Installation -------------------------
item centos8   Install CentOS 8
item ubuntu20  Install Ubuntu 20.04
item --gap -- ------------------------- Utilities -----------------------------
item memtest   Run Memtest86+
item shell     iPXE Shell
item exit      Exit to BIOS
choose --timeout 60000 --default exit selected || goto exit

goto ${selected}

:centos8
echo Booting CentOS 8 installer...
kernel http://${next-server}:${http-port}/images/centos8/vmlinuz initrd=initrd.img inst.repo=http://${next-server}:${http-port}/images/centos8 ip=dhcp
initrd http://${next-server}:${http-port}/images/centos8/initrd.img
boot || goto failed

:ubuntu20
echo Booting Ubuntu 20.04 installer...
kernel http://${next-server}:${http-port}/images/ubuntu20/vmlinuz initrd=initrd ip=dhcp url=http://${next-server}:${http-port}/images/ubuntu20/ubuntu.iso autoinstall
initrd http://${next-server}:${http-port}/images/ubuntu20/initrd
boot || goto failed

:memtest
echo Booting Memtest86+...
kernel http://${next-server}:${http-port}/boot/memtest86.bin
boot || goto failed

:shell
echo Dropping to iPXE shell...
shell || goto menu

:exit
echo Exiting to BIOS...
exit

:failed
echo Boot failed, dropping to shell
shell
"""

        # Write default script
        with open(self.ipxe_root / "scripts" / "default.ipxe", "w") as f:
            f.write(default_script)

    def add_os_image(self, os_name: str, iso_path: str) -> bool:
        """Add OS image for provisioning.

        Args:
            os_name: Operating system name
            iso_path: Path to ISO image

        Returns:
            bool: True if successful
        """
        try:
            if not os.path.exists(iso_path):
                Logger.error(f"ISO file not found: {iso_path}")
                return False

            # Create directory for OS
            os_dir = self.ipxe_root / "images" / os_name
            os.makedirs(os_dir, exist_ok=True)

            # Copy ISO to images directory
            shutil.copy(iso_path, os_dir / f"{os_name}.iso")

            Logger.info(f"Added OS image for {os_name}")
            return True

        except Exception as e:
            Logger.error(f"Failed to add OS image: {str(e)}")
            return False

    def extract_boot_files(self, os_name: str) -> bool:
        """Extract boot files from ISO image.

        Args:
            os_name: Operating system name

        Returns:
            bool: True if successful
        """
        try:
            iso_path = self.ipxe_root / "images" / os_name / f"{os_name}.iso"
            if not os.path.exists(iso_path):
                Logger.error(f"ISO file not found: {iso_path}")
                return False

            os_dir = self.ipxe_root / "images" / os_name

            # Use 7z to extract boot files
            if os_name.startswith("centos") or os_name.startswith("rhel"):
                # Extract kernel and initrd for CentOS/RHEL
                subprocess.run([
                    "7z", "e", str(iso_path),
                    "-o" + str(os_dir),
                    "isolinux/vmlinuz", "isolinux/initrd.img"
                ], check=True)

            elif os_name.startswith("ubuntu") or os_name.startswith("debian"):
                # Extract kernel and initrd for Ubuntu/Debian
                subprocess.run([
                    "7z", "e", str(iso_path),
                    "-o" + str(os_dir),
                    "casper/vmlinuz", "casper/initrd"
                ], check=True)

            Logger.info(f"Extracted boot files for {os_name}")
            return True

        except Exception as e:
            Logger.error(f"Failed to extract boot files: {str(e)}")
            return False

    def create_kickstart_file(self, os_name: str, config: Dict[str, Any]) -> bool:
        """Create kickstart file for automated installation.

        Args:
            os_name: Operating system name
            config: Kickstart configuration

        Returns:
            bool: True if successful
        """
        try:
            os_dir = self.ipxe_root / "images" / os_name
            os.makedirs(os_dir, exist_ok=True)

            # Basic kickstart template for CentOS/RHEL
            if os_name.startswith("centos") or os_name.startswith("rhel"):
                ks_content = f"""# Kickstart file for {os_name}
# Generated by Redfish Desktop

# System language
lang {config.get('lang', 'en_US.UTF-8')}

# Keyboard layouts
keyboard {config.get('keyboard', 'us')}

# Network information
network --bootproto={config.get('network_type', 'dhcp')} --device={config.get('network_device', 'eth0')} --onboot=yes

# Root password
rootpw --iscrypted {config.get('root_password_hash', '$6$rounds=10000$randomsalt$randomhash')}

# System timezone
timezone {config.get('timezone', 'America/New_York')} --isUtc

# Use graphical install
{'' if config.get('text_mode', False) else 'graphical'}

# System bootloader configuration
bootloader --location=mbr --boot-drive={config.get('boot_drive', 'sda')}

# Partition clearing information
clearpart --all --initlabel

# Disk partitioning information
autopart

# System services
services --enabled="chronyd"

# System authorization information
auth --enableshadow --passalgo=sha512

# Reboot after installation
reboot

# Packages
%packages
@^{config.get('environment', 'minimal')}
%end

# Post-installation script
%post
echo "Redfish Desktop automated installation completed"
%end
"""

                # Write kickstart file
                with open(os_dir / "ks.cfg", "w") as f:
                    f.write(ks_content)

            # Preseed file for Ubuntu/Debian
            elif os_name.startswith("ubuntu") or os_name.startswith("debian"):
                preseed_content = f"""# Preseed file for {os_name}
# Generated by Redfish Desktop

# Localization
d-i debian-installer/locale string {config.get('lang', 'en_US.UTF-8')}
d-i keyboard-configuration/xkb-keymap select {config.get('keyboard', 'us')}

# Network configuration
d-i netcfg/choose_interface select {config.get('network_device', 'auto')}
d-i netcfg/get_hostname string {config.get('hostname', 'redfish-client')}
d-i netcfg/get_domain string {config.get('domain', 'local')}

# Mirror settings
d-i mirror/country string manual
d-i mirror/http/hostname string archive.ubuntu.com
d-i mirror/http/directory string /ubuntu

# Account setup
d-i passwd/root-login boolean true
d-i passwd/root-password-crypted password {config.get('root_password_hash', '$6$rounds=10000$randomsalt$randomhash')}

# Clock and time zone setup
d-i clock-setup/utc boolean true
d-i time/zone string {config.get('timezone', 'America/New_York')}

# Partitioning
d-i partman-auto/method string regular
d-i partman-auto/choose_recipe select atomic
d-i partman-partitioning/confirm_write_new_label boolean true
d-i partman/choose_partition select finish
d-i partman/confirm boolean true
d-i partman/confirm_nooverwrite boolean true

# Package selection
tasksel tasksel/first multiselect {config.get('environment', 'standard')}

# Boot loader installation
d-i grub-installer/only_debian boolean true
d-i grub-installer/with_other_os boolean true
d-i grub-installer/bootdev string default

# Finishing up the installation
d-i finish-install/reboot_in_progress note
"""

                # Write preseed file
                with open(os_dir / "preseed.cfg", "w") as f:
                    f.write(preseed_content)

            Logger.info(f"Created automated installation file for {os_name}")
            return True

        except Exception as e:
            Logger.error(f"Failed to create automated installation file: {str(e)}")
            return False

    def create_custom_ipxe_script(self, script_name: str, content: str) -> bool:
        """Create custom iPXE script.

        Args:
            script_name: Script name
            content: Script content

        Returns:
            bool: True if successful
        """
        try:
            script_path = self.ipxe_root / "scripts" / f"{script_name}.ipxe"

            with open(script_path, "w") as f:
                f.write(content)

            Logger.info(f"Created custom iPXE script: {script_name}")
            return True

        except Exception as e:
            Logger.error(f"Failed to create custom iPXE script: {str(e)}")
            return False

    def start_http_server(self, port: int = 8000) -> bool:
        """Start HTTP server for iPXE files.

        Args:
            port: HTTP port

        Returns:
            bool: True if successful
        """
        try:
            self.http_port = port

            # Create HTTP server
            handler = http.server.SimpleHTTPRequestHandler

            class IPXEHTTPServer(socketserver.TCPServer):
                allow_reuse_address = True

            # Change to iPXE root directory
            os.chdir(self.ipxe_root)

            self.http_server = IPXEHTTPServer(("", port), handler)

            # Start server in a separate thread
            self.server_thread = threading.Thread(target=self.http_server.serve_forever)
            self.server_thread.daemon = True
            self.server_thread.start()

            Logger.info(f"Started HTTP server on port {port}")
            self.running = True
            return True

        except Exception as e:
            Logger.error(f"Failed to start HTTP server: {str(e)}")
            return False

    def stop_servers(self) -> bool:
        """Stop all running servers.

        Returns:
            bool: True if successful
        """
        try:
            if self.http_server:
                self.http_server.shutdown()
                self.http_server = None

            if self.tftp_server:
                self.tftp_server.shutdown()
                self.tftp_server = None

            self.running = False
            Logger.info("Stopped all provisioning servers")
            return True

        except Exception as e:
            Logger.error(f"Failed to stop servers: {str(e)}")
            return False

    def is_running(self) -> bool:
        """Check if servers are running.

        Returns:
            bool: True if running
        """
        return self.running



import os
import json
from typing import Dict, Any, Optional
import appdirs

# Define default settings
DEFAULT_SETTINGS = {
    "theme": "light",
    "connection_profiles": [],
    "auto_reconnect": False,
    "refresh_interval": 30
}

class Settings:
    """Settings manager for the application."""

    def __init__(self):
        """Initialize settings."""
        # Determine config directory
        self.config_dir = appdirs.user_config_dir("RedFishDesktop")
        self.config_file = os.path.join(self.config_dir, "settings.json")

        # Create config directory if it doesn't exist
        if not os.path.exists(self.config_dir):
            os.makedirs(self.config_dir)

        # Load settings
        self.settings = self.load_settings()

    def load_settings(self) -> Dict[str, Any]:
        """Load settings from file."""
        if os.path.exists(self.config_file):
            try:
                with open(self.config_file, "r") as f:
                    return json.load(f)
            except (json.JSONDecodeError, IOError):
                # If error loading settings, use defaults
                return DEFAULT_SETTINGS.copy()
        else:
            # If file doesn't exist, use defaults
            return DEFAULT_SETTINGS.copy()

    def save_settings(self) -> None:
        """Save settings to file."""
        try:
            with open(self.config_file, "w") as f:
                json.dump(self.settings, f, indent=2)
        except IOError:
            # Ignore errors saving settings
            pass

    def get(self, key: str, default: Any = None) -> Any:
        """Get a setting value."""
        return self.settings.get(key, default)

    def set(self, key: str, value: Any) -> None:
        """Set a setting value."""
        self.settings[key] = value
        self.save_settings()

    def get_theme(self) -> str:
        """Get the current theme."""
        return self.settings.get("theme", "light")

    def set_theme(self, theme: str) -> None:
        """Set the current theme."""
        self.settings["theme"] = theme
        self.save_settings()

    def get_auto_reconnect(self) -> bool:
        """Get auto-reconnect setting."""
        return self.settings.get("auto_reconnect", False)

    def set_auto_reconnect(self, value: bool) -> None:
        """Set auto-reconnect setting."""
        self.settings["auto_reconnect"] = value
        self.save_settings()

    def get_refresh_interval(self) -> int:
        """Get refresh interval in seconds."""
        return self.settings.get("refresh_interval", 30)

    def set_refresh_interval(self, value: int) -> None:
        """Set refresh interval in seconds."""
        self.settings["refresh_interval"] = value
        self.save_settings()

    def get_connection_profiles(self) -> list:
        """Get saved connection profiles."""
        return self.settings.get("connection_profiles", [])

    def add_connection_profile(self, profile: Dict[str, Any]) -> None:
        """Add a connection profile."""
        profiles = self.get_connection_profiles()

        # Replace profile if it already exists (by name)
        for i, p in enumerate(profiles):
            if p.get("name") == profile.get("name"):
                profiles[i] = profile
                break
        else:
            # Add new profile
            profiles.append(profile)

        self.settings["connection_profiles"] = profiles
        self.save_settings()

    def remove_connection_profile(self, name: str) -> None:
        """Remove a connection profile."""
        profiles = self.get_connection_profiles()

        # Remove profile if it exists
        self.settings["connection_profiles"] = [p for p in profiles if p.get("name") != name]
        self.save_settings()

# Create a global settings instance
settings = Settings()



from PyQt6.QtWidgets import QWidget, QVBoxLayout, QLabel, QScrollArea
from PyQt6.QtCore import Qt

from greenfish.utils.logger import Logger


class BaseMetricsPanel(QWidget):
    """Base class for all metrics panels"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.client = None
        self.resource_id = None
        self.data = None
        self.initUI()

    def initUI(self):
        """Initialize the UI components"""
        layout = QVBoxLayout()

        # Create a scroll area for the content
        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        scroll.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)

        # Content widget that will contain all the metrics
        self.content_widget = QWidget()
        self.content_layout = QVBoxLayout(self.content_widget)

        # Status label for showing loading/error messages
        self.status_label = QLabel("No data available")
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.content_layout.addWidget(self.status_label)

        scroll.setWidget(self.content_widget)
        layout.addWidget(scroll)

        self.setLayout(layout)

    def set_client(self, client, resource_id=None):
        """Set the Redfish client and resource ID"""
        self.client = client
        self.resource_id = resource_id

    def refresh(self):
        """Refresh the metrics data - to be implemented by subclasses"""
        raise NotImplementedError("Subclasses must implement refresh()")

    def clear_content(self):
        """Clear all content from the panel"""
        # Remove all widgets except the status label
        while self.content_layout.count() > 1:
            item = self.content_layout.takeAt(0)
            if item.widget():
                item.widget().deleteLater()

    def show_loading(self):
        """Show loading message"""
        self.status_label.setText("Loading data...")
        self.status_label.setVisible(True)

    def show_error(self, message):
        """Show error message"""
        self.status_label.setText(f"Error: {message}")
        self.status_label.setVisible(True)

    def hide_status(self):
        """Hide the status label"""
        self.status_label.setVisible(False)

    def create_section(self, title):
        """Create a section with a title"""
        section = QWidget()
        section_layout = QVBoxLayout(section)
        section_layout.setContentsMargins(0, 10, 0, 10)

        # Section title
        title_label = QLabel(title)
        title_label.setStyleSheet("font-weight: bold; font-size: 14px;")
        section_layout.addWidget(title_label)

        return section, section_layout



from PyQt6.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QLabel,
                             QPushButton, QTableWidget, QTableWidgetItem, QHeaderView,
                             QComboBox, QLineEdit, QCheckBox, QMessageBox, QTabWidget,
                             QSpinBox, QGroupBox, QScrollArea)
from PyQt6.QtCore import Qt
from typing import Dict, Any, Optional

class BiosConfigWidget(QWidget):
    """Widget for displaying and modifying BIOS settings."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.bios_data = {}
        self.modified_attributes = {}
        self.setup_ui()

    def setup_ui(self):
        """Setup the user interface."""
        main_layout = QVBoxLayout(self)

        # Create tab widget for different setting categories
        self.tabs = QTabWidget()
        main_layout.addWidget(self.tabs)

        # Add default "All Settings" tab
        self.all_settings_tab = QWidget()
        self.all_settings_layout = QVBoxLayout(self.all_settings_tab)

        # Create table for BIOS settings
        self.settings_table = QTableWidget()
        self.settings_table.setColumnCount(3)
        self.settings_table.setHorizontalHeaderLabels(["Setting", "Value", "Description"])
        self.settings_table.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        self.settings_table.horizontalHeader().setSectionResizeMode(1, QHeaderView.ResizeMode.ResizeToContents)
        self.all_settings_layout.addWidget(self.settings_table)

        self.tabs.addTab(self.all_settings_tab, "All Settings")

        # Pending settings tab
        self.pending_tab = QWidget()
        self.pending_layout = QVBoxLayout(self.pending_tab)

        self.pending_table = QTableWidget()
        self.pending_table.setColumnCount(3)
        self.pending_table.setHorizontalHeaderLabels(["Setting", "Pending Value", "Current Value"])
        self.pending_table.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        self.pending_layout.addWidget(self.pending_table)

        self.tabs.addTab(self.pending_tab, "Pending Changes")

        # Buttons row
        button_layout = QHBoxLayout()

        self.refresh_button = QPushButton("Refresh")
        button_layout.addWidget(self.refresh_button)

        self.apply_button = QPushButton("Apply Changes")
        self.apply_button.setEnabled(False)
        button_layout.addWidget(self.apply_button)

        self.reset_button = QPushButton("Reset")
        self.reset_button.setEnabled(False)
        button_layout.addWidget(self.reset_button)

        main_layout.addLayout(button_layout)

        # Status label
        self.status_label = QLabel("No BIOS data loaded")
        main_layout.addWidget(self.status_label)

        # Connect signals
        self.refresh_button.clicked.connect(self.refresh_clicked)
        self.apply_button.clicked.connect(self.apply_changes)
        self.reset_button.clicked.connect(self.reset_changes)

        # Set styling
        self.setStyleSheet("""
            QTableWidget {
                background-color: rgb(255, 255, 255);
                border: 1px solid #CCCCCC;
                border-radius: 3px;
            }
            QTableWidget::item {
                padding: 4px;
            }
            QHeaderView::section {
                background-color: rgb(240, 240, 220);
                padding: 4px;
                border: 1px solid #CCCCCC;
                border-left: none;
                border-top: none;
            }
            QPushButton {
                padding: 5px 15px;
                background-color: rgb(200, 200, 180);
                border: 1px solid #CCCCCC;
                border-radius: 3px;
            }
            QPushButton:hover {
                background-color: rgb(180, 180, 160);
            }
            QTabWidget::pane {
                border: 1px solid #CCCCCC;
                border-radius: 3px;
            }
            QTabBar::tab {
                background-color: rgb(240, 240, 220);
                border: 1px solid #CCCCCC;
                border-bottom: none;
                border-top-left-radius: 3px;
                border-top-right-radius: 3px;
                padding: 5px 10px;
                margin-right: 2px;
            }
            QTabBar::tab:selected {
                background-color: rgb(255, 255, 220);
                border-bottom: 1px solid rgb(255, 255, 220);
            }
            QTabBar::tab:!selected {
                margin-top: 2px;
            }
        """)

    def load_bios_data(self, bios_data: Dict[str, Any]):
        """Load BIOS data into the widget."""
        self.bios_data = bios_data
        self.modified_attributes = {}  # Reset modified attributes

        # Clear any existing category tabs (except first two)
        while self.tabs.count() > 2:
            self.tabs.removeTab(2)

        # Update settings table
        self.update_settings_table()

        # Check if we have registry data for categories
        registry = bios_data.get("registry", None)
        if registry and "RegistryEntries" in registry:
            # Create tabs for each category
            categories = {}

            # Group attributes by category
            registry_entries = registry["RegistryEntries"]
            if "Attributes" in registry_entries:
                for attr in registry_entries["Attributes"]:
                    if "AttributeName" in attr and "MenuPath" in attr:
                        category = attr.get("MenuPath", "Other")
                        if category not in categories:
                            categories[category] = []
                        categories[category].append(attr)

            # Create a tab for each category
            for category, attrs in categories.items():
                # Skip if empty
                if not attrs:
                    continue

                # Create tab
                tab = QWidget()
                tab_layout = QVBoxLayout(tab)

                # Create scroll area for potentially long lists
                scroll = QScrollArea()
                scroll.setWidgetResizable(True)
                scroll_content = QWidget()
                scroll_layout = QVBoxLayout(scroll_content)

                # Add attributes to this category
                for attr in attrs:
                    attr_name = attr.get("AttributeName")
                    attr_display = attr.get("DisplayName", attr_name)
                    attr_desc = attr.get("HelpText", "")
                    attr_type = attr.get("Type", "String")

                    # Get current value
                    current_value = self.bios_data["settings"].get("Attributes", {}).get(attr_name, "N/A")

                    # Create a group box for this setting
                    group = QGroupBox(attr_display)
                    group_layout = QVBoxLayout(group)

                    # Add description
                    if attr_desc:
                        desc_label = QLabel(attr_desc)
                        desc_label.setWordWrap(True)
                        group_layout.addWidget(desc_label)

                    # Create editor based on type
                    editor = None

                    if attr_type == "Enumeration" and "Value" in attr:
                        # Create combobox for enumeration
                        editor = QComboBox()
                        for val in attr["Value"]:
                            value_name = val.get("ValueName")
                            value_display = val.get("ValueDisplayName", value_name)
                            editor.addItem(value_display, value_name)

                        # Set current value
                        index = editor.findData(current_value)
                        if index >= 0:
                            editor.setCurrentIndex(index)

                        editor.currentIndexChanged.connect(
                            lambda index, attr=attr_name, combo=editor: self.setting_changed(attr, combo.currentData())
                        )

                    elif attr_type == "Boolean":
                        # Create checkbox for boolean
                        editor = QCheckBox("Enabled")
                        editor.setChecked(current_value)

                        editor.stateChanged.connect(
                            lambda state, attr=attr_name: self.setting_changed(attr, state == Qt.CheckState.Checked)
                        )

                    elif attr_type == "Integer":
                        # Create spinbox for integer
                        editor = QSpinBox()

                        # Set range if available
                        if "LowerBound" in attr:
                            editor.setMinimum(attr["LowerBound"])
                        if "UpperBound" in attr:
                            editor.setMaximum(attr["UpperBound"])

                        # Set current value
                        try:
                            editor.setValue(int(current_value))
                        except (ValueError, TypeError):
                            pass

                        editor.valueChanged.connect(
                            lambda value, attr=attr_name: self.setting_changed(attr, value)
                        )

                    else:
                        # Default to text input for other types
                        editor = QLineEdit(str(current_value))

                        editor.textChanged.connect(
                            lambda text, attr=attr_name: self.setting_changed(attr, text)
                        )

                    # Add editor to group
                    if editor:
                        editor_layout = QHBoxLayout()
                        value_label = QLabel("Value:")
                        editor_layout.addWidget(value_label)
                        editor_layout.addWidget(editor)
                        group_layout.addLayout(editor_layout)

                    # Add to scroll layout
                    scroll_layout.addWidget(group)

                # Add spacer at the end
                scroll_layout.addStretch()

                # Set scroll content
                scroll.setWidget(scroll_content)
                tab_layout.addWidget(scroll)

                # Add tab
                self.tabs.addTab(tab, category)

        # Update status
        self.status_label.setText(f"Loaded BIOS settings ({len(self.bios_data.get('settings', {}).get('Attributes', {}))} attributes)")

    def update_settings_table(self):
        """Update the settings table with current BIOS data."""
        settings = self.bios_data.get("settings", {}).get("Attributes", {})

        # Update main settings table
        self.settings_table.setRowCount(len(settings))

        row = 0
        for name, value in settings.items():
            # Setting name
            name_item = QTableWidgetItem(name)
            name_item.setFlags(name_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
            self.settings_table.setItem(row, 0, name_item)

            # Setting value (editable)
            value_item = QTableWidgetItem(str(value))
            self.settings_table.setItem(row, 1, value_item)

            # Description (from registry if available)
            desc = self.get_attribute_description(name)
            desc_item = QTableWidgetItem(desc)
            desc_item.setFlags(desc_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
            self.settings_table.setItem(row, 2, desc_item)

            row += 1

        # Connect to item changed signal
        self.settings_table.itemChanged.connect(self.table_item_changed)

    def update_pending_table(self, pending_settings: Dict[str, Any]):
        """Update the pending settings table."""
        current_settings = self.bios_data.get("settings", {}).get("Attributes", {})
        pending_attributes = pending_settings.get("Attributes", {})

        # Clear table
        self.pending_table.setRowCount(len(pending_attributes))

        row = 0
        for name, value in pending_attributes.items():
            # Only show if different from current
            current_value = current_settings.get(name, "N/A")
            if str(value) != str(current_value):
                # Setting name
                name_item = QTableWidgetItem(name)
                name_item.setFlags(name_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
                self.pending_table.setItem(row, 0, name_item)

                # Pending value
                pending_item = QTableWidgetItem(str(value))
                pending_item.setFlags(pending_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
                self.pending_table.setItem(row, 1, pending_item)

                # Current value
                current_item = QTableWidgetItem(str(current_value))
                current_item.setFlags(current_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
                self.pending_table.setItem(row, 2, current_item)

                row += 1

        # Update actual row count (in case we skipped some)
        self.pending_table.setRowCount(row)

    def get_attribute_description(self, attr_name: str) -> str:
        """Get attribute description from registry if available."""
        registry = self.bios_data.get("registry", None)
        if registry and "RegistryEntries" in registry:
            registry_entries = registry["RegistryEntries"]
            if "Attributes" in registry_entries:
                for attr in registry_entries["Attributes"]:
                    if attr.get("AttributeName") == attr_name:
                        return attr.get("HelpText", "")
        return ""

    def table_item_changed(self, item):
        """Handle changes to table items."""
        # Only process value column changes (column 1)
        if item.column() == 1:
            row = item.row()
            attr_name = self.settings_table.item(row, 0).text()
            attr_value = item.text()

            # Convert value based on registry type if possible
            converted_value = self.convert_value_by_type(attr_name, attr_value)

            # Update modified attributes
            self.setting_changed(attr_name, converted_value)

    def convert_value_by_type(self, attr_name: str, value: str) -> Any:
        """Convert string value to appropriate type based on registry."""
        # Get attribute type from registry
        registry = self.bios_data.get("registry", None)
        if registry and "RegistryEntries" in registry:
            registry_entries = registry["RegistryEntries"]
            if "Attributes" in registry_entries:
                for attr in registry_entries["Attributes"]:
                    if attr.get("AttributeName") == attr_name:
                        attr_type = attr.get("Type", "String")

                        if attr_type == "Boolean":
                            return value.lower() in ("true", "1", "yes", "y", "on")
                        elif attr_type == "Integer":
                            try:
                                return int(value)
                            except ValueError:
                                pass
                        # Enumeration and other types as string
                        return value

        # Default to string if no type info
        return value

    def setting_changed(self, attr_name: str, value: Any):
        """Handle setting changes."""
        original_value = self.bios_data.get("settings", {}).get("Attributes", {}).get(attr_name)

        if str(value) != str(original_value):
            # Value changed, add to modified attributes
            self.modified_attributes[attr_name] = value
        else:
            # Value unchanged, remove from modified attributes
            if attr_name in self.modified_attributes:
                del self.modified_attributes[attr_name]

        # Update button states
        self.apply_button.setEnabled(len(self.modified_attributes) > 0)
        self.reset_button.setEnabled(len(self.modified_attributes) > 0)

        # Update status
        if len(self.modified_attributes) > 0:
            self.status_label.setText(f"Modified {len(self.modified_attributes)} settings")
        else:
            self.status_label.setText("No pending changes")

    def refresh_clicked(self):
        """Signal to refresh BIOS data."""
        # This will be connected to the parent widget's refresh method
        pass

    def apply_changes(self):
        """Signal to apply BIOS changes."""
        # This will be connected to the parent widget's apply method
        pass

    def reset_changes(self):
        """Reset all pending changes."""
        self.modified_attributes = {}

        # Reload BIOS data to reset UI
        self.load_bios_data(self.bios_data)

        # Update status
        self.status_label.setText("Changes reset")

        # Disable buttons
        self.apply_button.setEnabled(False)
        self.reset_button.setEnabled(False)

    def get_modified_attributes(self) -> Dict[str, Any]:
        """Get the modified attributes."""
        return self.modified_attributes.copy()



import uuid
from datetime import datetime
from typing import Dict, Any


class ConnectionProfile:
    """Class representing a saved connection profile."""

    def __init__(self, name: str, base_url: str, username: str, password: str = None,
                auth_type: str = "session", description: str = "", id: str = None):
        """Initialize a connection profile."""
        self.id = id or str(uuid.uuid4())
        self.name = name
        self.base_url = base_url
        self.username = username
        self.password = password  # Note: Password should be encrypted in a real app
        self.auth_type = auth_type
        self.description = description
        self.created = datetime.now().isoformat()
        self.last_used = None

    def to_dict(self, include_password: bool = False) -> Dict[str, Any]:
        """Convert profile to dictionary."""
        result = {
            "id": self.id,
            "name": self.name,
            "base_url": self.base_url,
            "username": self.username,
            "auth_type": self.auth_type,
            "description": self.description,
            "created": self.created,
            "last_used": self.last_used
        }

        if include_password and self.password:
            result["password"] = self.password

        return result

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ConnectionProfile':
        """Create profile from dictionary."""
        profile = cls(
            name=data.get("name", ""),
            base_url=data.get("base_url", ""),
            username=data.get("username", ""),
            password=data.get("password"),
            auth_type=data.get("auth_type", "session"),
            description=data.get("description", ""),
            id=data.get("id")
        )

        profile.created = data.get("created", datetime.now().isoformat())
        profile.last_used = data.get("last_used")

        return profile



from PyQt6.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton,
                             QLabel, QGroupBox, QTableWidget, QTableWidgetItem,
                             QMessageBox, QCheckBox)
from PyQt6.QtCore import Qt
from typing import Dict, Any, Optional
from greenfish.utils.logger import logger

class SecureBootWidget(QWidget):
    """Widget for managing Secure Boot configuration."""

    def __init__(self, redfish_client=None, parent=None):
        super().__init__(parent)
        self.redfish_client = redfish_client

        # Create layout
        layout = QVBoxLayout(self)

        # Status group
        status_group = QGroupBox("Secure Boot Status")
        status_layout = QVBoxLayout(status_group)

        # Enable/disable checkbox
        self.enable_check = QCheckBox("Enable Secure Boot")
        self.enable_check.stateChanged.connect(self.on_enable_changed)
        status_layout.addWidget(self.enable_check)

        # Current state label
        self.state_label = QLabel()
        status_layout.addWidget(self.state_label)

        layout.addWidget(status_group)

        # Key management group
        keys_group = QGroupBox("Key Management")
        keys_layout = QVBoxLayout(keys_group)

        # Key management buttons
        button_layout = QHBoxLayout()

        self.reset_button = QPushButton("Reset Keys to Default")
        self.reset_button.clicked.connect(self.reset_keys)
        button_layout.addWidget(self.reset_button)

        self.clear_button = QPushButton("Clear All Keys")
        self.clear_button.clicked.connect(self.clear_keys)
        button_layout.addWidget(self.clear_button)

        keys_layout.addLayout(button_layout)

        layout.addWidget(keys_group)

        # Database group
        db_group = QGroupBox("Certificate Databases")
        db_layout = QVBoxLayout(db_group)

        # Database table
        self.db_table = QTableWidget()
        self.db_table.setColumnCount(3)
        self.db_table.setHorizontalHeaderLabels(["Database", "Type", "Certificates"])
        self.db_table.setSelectionBehavior(QTableWidget.SelectionBehavior.SelectRows)
        self.db_table.setSelectionMode(QTableWidget.SelectionMode.SingleSelection)
        self.db_table.verticalHeader().setVisible(False)
        db_layout.addWidget(self.db_table)

        layout.addWidget(db_group)

        # Refresh button
        self.refresh_button = QPushButton("Refresh")
        self.refresh_button.clicked.connect(self.refresh_data)
        layout.addWidget(self.refresh_button)

        # Apply styling
        self.setStyleSheet("""
            QWidget {
                background-color: rgb(255, 255, 220);
            }
            QGroupBox {
                font-weight: bold;
                border: 1px solid #CCCCCC;
                border-radius: 5px;
                margin-top: 10px;
                padding-top: 15px;
            }
            QTableWidget {
                background-color: white;
                alternate-background-color: rgb(250, 250, 240);
            }
            QPushButton {
                padding: 5px 15px;
                background-color: rgb(200, 200, 180);
                border: 1px solid #CCCCCC;
                border-radius: 3px;
            }
            QPushButton:hover {
                background-color: rgb(180, 180, 160);
            }
        """)

    def set_redfish_client(self, client):
        """Set the Redfish client."""
        self.redfish_client = client
        self.refresh_data()

    def refresh_data(self):
        """Refresh Secure Boot data."""
        if not self.redfish_client:
            return

        try:
            # Get Secure Boot information
            secure_boot = self.redfish_client.get_secure_boot()

            # Update enable checkbox
            enabled = secure_boot.get("SecureBootEnable", False)
            self.enable_check.setChecked(enabled)

            # Update state label
            current_state = secure_boot.get("SecureBootCurrentBoot", "Unknown")
            mode = secure_boot.get("SecureBootMode", "Unknown")
            self.state_label.setText(
                f"Current State: {current_state}\n"
                f"Mode: {mode}"
            )

            # Get database information
            try:
                databases = self.redfish_client.get_secure_boot_databases()

                # Update database table
                self.db_table.setRowCount(len(databases))
                for row, db in enumerate(databases):
                    self.db_table.setItem(row, 0, QTableWidgetItem(db.get("Name", "")))
                    self.db_table.setItem(row, 1, QTableWidgetItem(db.get("DatabaseId", "")))
                    cert_count = len(db.get("Certificates", {}).get("Members", []))
                    self.db_table.setItem(row, 2, QTableWidgetItem(str(cert_count)))

                # Adjust column widths
                self.db_table.resizeColumnsToContents()

            except Exception as e:
                logger.warning(f"Failed to get Secure Boot databases: {str(e)}")
                self.db_table.setRowCount(0)

            logger.success("Secure Boot data refreshed successfully")

        except Exception as e:
            logger.error(f"Failed to refresh Secure Boot data: {str(e)}")
            QMessageBox.critical(self, "Error", str(e))

    def on_enable_changed(self, state):
        """Handle Secure Boot enable/disable."""
        if not self.redfish_client:
            return

        try:
            enable = state == Qt.CheckState.Checked.value

            # Confirm action
            confirm = QMessageBox.question(
                self,
                "Confirm Secure Boot Change",
                f"Are you sure you want to {'enable' if enable else 'disable'} Secure Boot?\n\n"
                "Note: This change requires a system reboot to take effect.",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
            )

            if confirm != QMessageBox.StandardButton.Yes:
                # Revert checkbox state
                self.enable_check.setChecked(not enable)
                return

            # Update Secure Boot state
            self.redfish_client.enable_secure_boot(enable)

            # Show reboot message
            QMessageBox.information(
                self,
                "Secure Boot Updated",
                "Secure Boot state updated successfully.\n\n"
                "A system reboot is required for changes to take effect."
            )

        except Exception as e:
            logger.error(f"Failed to update Secure Boot state: {str(e)}")
            QMessageBox.critical(self, "Error", str(e))
            # Revert checkbox state
            self.enable_check.setChecked(not enable)

    def reset_keys(self):
        """Reset Secure Boot keys to default."""
        if not self.redfish_client:
            return

        try:
            # Confirm action
            confirm = QMessageBox.question(
                self,
                "Confirm Reset Keys",
                "Are you sure you want to reset all Secure Boot keys to their default values?\n\n"
                "This will remove any custom keys and restore the original manufacturer keys.",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
            )

            if confirm != QMessageBox.StandardButton.Yes:
                return

            # Reset keys
            self.redfish_client.reset_secure_boot()

            # Refresh data
            self.refresh_data()

            QMessageBox.information(
                self,
                "Keys Reset",
                "Secure Boot keys have been reset to their default values."
            )

        except Exception as e:
            logger.error(f"Failed to reset Secure Boot keys: {str(e)}")
            QMessageBox.critical(self, "Error", str(e))

    def clear_keys(self):
        """Clear all Secure Boot keys."""
        if not self.redfish_client:
            return

        try:
            # Confirm action
            confirm = QMessageBox.question(
                self,
                "Confirm Clear Keys",
                "Are you sure you want to clear all Secure Boot keys?\n\n"
                "WARNING: This will delete all keys and may prevent the system from booting!",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
            )

            if confirm != QMessageBox.StandardButton.Yes:
                return

            # Clear keys
            self.redfish_client.clear_secure_boot_keys()

            # Refresh data
            self.refresh_data()

            QMessageBox.warning(
                self,
                "Keys Cleared",
                "All Secure Boot keys have been cleared.\n\n"
                "WARNING: The system may not boot until new keys are installed!"
            )

        except Exception as e:
            logger.error(f"Failed to clear Secure Boot keys: {str(e)}")
            QMessageBox.critical(self, "Error", str(e))



"""OS deployment module for bare metal provisioning."""
from typing import Dict, Any, Optional, List, Union
import os
import json
import time
import shutil
from pathlib import Path
import tempfile
import uuid

from greenfish.utils.logger import Logger
from greenfish.core.provisioning.ipxe import IPXEClient

class OSDeployment:
    """OS deployment for bare metal provisioning."""

    def __init__(self, ipxe_client: IPXEClient = None):
        """Initialize OS deployment.

        Args:
            ipxe_client: iPXE client instance
        """
        self.ipxe_client = ipxe_client or IPXEClient()
        self.deployment_dir = None
        self.deployments = {}

    def setup_deployment_environment(self, deployment_dir: str) -> bool:
        """Set up deployment environment.

        Args:
            deployment_dir: Directory for deployment files

        Returns:
            bool: True if setup successful
        """
        try:
            self.deployment_dir = Path(deployment_dir).resolve()

            # Create directory structure if it doesn't exist
            os.makedirs(self.deployment_dir / "templates", exist_ok=True)
            os.makedirs(self.deployment_dir / "configs", exist_ok=True)
            os.makedirs(self.deployment_dir / "logs", exist_ok=True)

            # Create default templates
            self._create_default_templates()

            # Load existing deployments
            self._load_deployments()

            Logger.info(f"Deployment environment set up at {self.deployment_dir}")
            return True

        except Exception as e:
            Logger.error(f"Failed to set up deployment environment: {str(e)}")
            return False

    def _create_default_templates(self) -> None:
        """Create default deployment templates."""
        # CentOS template
        centos_template = {
            "name": "CentOS 8 Server",
            "os_type": "centos8",
            "description": "CentOS 8 minimal server installation",
            "config": {
                "lang": "en_US.UTF-8",
                "keyboard": "us",
                "network_type": "dhcp",
                "network_device": "eth0",
                "timezone": "America/New_York",
                "text_mode": False,
                "boot_drive": "sda",
                "environment": "minimal"
            }
        }

        # Ubuntu template
        ubuntu_template = {
            "name": "Ubuntu 20.04 Server",
            "os_type": "ubuntu20",
            "description": "Ubuntu 20.04 LTS server installation",
            "config": {
                "lang": "en_US.UTF-8",
                "keyboard": "us",
                "network_device": "auto",
                "hostname": "redfish-client",
                "domain": "local",
                "timezone": "America/New_York",
                "environment": "standard"
            }
        }

        # Save templates
        with open(self.deployment_dir / "templates" / "centos8.json", "w") as f:
            json.dump(centos_template, f, indent=2)

        with open(self.deployment_dir / "templates" / "ubuntu20.json", "w") as f:
            json.dump(ubuntu_template, f, indent=2)

    def _load_deployments(self) -> None:
        """Load existing deployments."""
        self.deployments = {}

        try:
            config_dir = self.deployment_dir / "configs"

            if not config_dir.exists():
                return

            for config_file in config_dir.glob("*.json"):
                try:
                    with open(config_file, "r") as f:
                        deployment = json.load(f)

                    deployment_id = config_file.stem
                    self.deployments[deployment_id] = deployment

                except Exception as e:
                    Logger.error(f"Failed to load deployment from {config_file}: {str(e)}")

        except Exception as e:
            Logger.error(f"Failed to load deployments: {str(e)}")

    def get_available_templates(self) -> List[Dict[str, Any]]:
        """Get available deployment templates.

        Returns:
            List of templates
        """
        templates = []

        try:
            template_dir = self.deployment_dir / "templates"

            if not template_dir.exists():
                return templates

            for template_file in template_dir.glob("*.json"):
                try:
                    with open(template_file, "r") as f:
                        template = json.load(f)

                    template["id"] = template_file.stem
                    templates.append(template)

                except Exception as e:
                    Logger.error(f"Failed to load template from {template_file}: {str(e)}")

        except Exception as e:
            Logger.error(f"Failed to get templates: {str(e)}")

        return templates

    def create_deployment(self, template_id: str, name: str, config_overrides: Dict[str, Any] = None) -> str:
        """Create a new deployment from a template.

        Args:
            template_id: Template ID
            name: Deployment name
            config_overrides: Configuration overrides

        Returns:
            str: Deployment ID
        """
        try:
            # Load template
            template_path = self.deployment_dir / "templates" / f"{template_id}.json"

            if not template_path.exists():
                Logger.error(f"Template not found: {template_id}")
                return ""

            with open(template_path, "r") as f:
                template = json.load(f)

            # Create deployment
            deployment_id = str(uuid.uuid4())

            deployment = {
                "id": deployment_id,
                "name": name,
                "template_id": template_id,
                "os_type": template.get("os_type", ""),
                "description": template.get("description", ""),
                "status": "created",
                "created_at": int(time.time()),
                "updated_at": int(time.time()),
                "config": template.get("config", {})
            }

            # Apply config overrides
            if config_overrides:
                deployment["config"].update(config_overrides)

            # Save deployment
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            with open(config_path, "w") as f:
                json.dump(deployment, f, indent=2)

            # Add to deployments
            self.deployments[deployment_id] = deployment

            Logger.info(f"Created deployment {name} with ID {deployment_id}")
            return deployment_id

        except Exception as e:
            Logger.error(f"Failed to create deployment: {str(e)}")
            return ""

    def update_deployment(self, deployment_id: str, updates: Dict[str, Any]) -> bool:
        """Update deployment configuration.

        Args:
            deployment_id: Deployment ID
            updates: Updates to apply

        Returns:
            bool: True if successful
        """
        try:
            if deployment_id not in self.deployments:
                Logger.error(f"Deployment not found: {deployment_id}")
                return False

            deployment = self.deployments[deployment_id]

            # Apply updates
            if "name" in updates:
                deployment["name"] = updates["name"]

            if "description" in updates:
                deployment["description"] = updates["description"]

            if "config" in updates:
                deployment["config"].update(updates["config"])

            deployment["updated_at"] = int(time.time())

            # Save deployment
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            with open(config_path, "w") as f:
                json.dump(deployment, f, indent=2)

            Logger.info(f"Updated deployment {deployment['name']}")
            return True

        except Exception as e:
            Logger.error(f"Failed to update deployment: {str(e)}")
            return False

    def delete_deployment(self, deployment_id: str) -> bool:
        """Delete a deployment.

        Args:
            deployment_id: Deployment ID

        Returns:
            bool: True if successful
        """
        try:
            if deployment_id not in self.deployments:
                Logger.error(f"Deployment not found: {deployment_id}")
                return False

            deployment = self.deployments[deployment_id]

            # Delete deployment file
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            if config_path.exists():
                os.remove(config_path)

            # Remove from deployments
            del self.deployments[deployment_id]

            Logger.info(f"Deleted deployment {deployment['name']}")
            return True

        except Exception as e:
            Logger.error(f"Failed to delete deployment: {str(e)}")
            return False

    def get_deployments(self) -> List[Dict[str, Any]]:
        """Get all deployments.

        Returns:
            List of deployments
        """
        return list(self.deployments.values())

    def get_deployment(self, deployment_id: str) -> Dict[str, Any]:
        """Get deployment by ID.

        Args:
            deployment_id: Deployment ID

        Returns:
            Dict: Deployment configuration
        """
        return self.deployments.get(deployment_id, {})

    def prepare_deployment(self, deployment_id: str, iso_path: str = None) -> bool:
        """Prepare deployment for execution.

        Args:
            deployment_id: Deployment ID
            iso_path: Path to ISO image (optional)

        Returns:
            bool: True if successful
        """
        try:
            if deployment_id not in self.deployments:
                Logger.error(f"Deployment not found: {deployment_id}")
                return False

            deployment = self.deployments[deployment_id]
            os_type = deployment.get("os_type", "")

            # Set up iPXE environment if not already set up
            if not self.ipxe_client.ipxe_root:
                ipxe_root = self.deployment_dir / "ipxe"
                self.ipxe_client.setup_ipxe_environment(ipxe_root)

            # Add OS image if provided
            if iso_path:
                self.ipxe_client.add_os_image(os_type, iso_path)
                self.ipxe_client.extract_boot_files(os_type)

            # Create kickstart/preseed file
            self.ipxe_client.create_kickstart_file(os_type, deployment["config"])

            # Update deployment status
            deployment["status"] = "prepared"
            deployment["updated_at"] = int(time.time())

            # Save deployment
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            with open(config_path, "w") as f:
                json.dump(deployment, f, indent=2)

            Logger.info(f"Prepared deployment {deployment['name']}")
            return True

        except Exception as e:
            Logger.error(f"Failed to prepare deployment: {str(e)}")
            return False

    def start_deployment(self, deployment_id: str, http_port: int = 8000) -> bool:
        """Start deployment by starting the HTTP server.

        Args:
            deployment_id: Deployment ID
            http_port: HTTP port

        Returns:
            bool: True if successful
        """
        try:
            if deployment_id not in self.deployments:
                Logger.error(f"Deployment not found: {deployment_id}")
                return False

            deployment = self.deployments[deployment_id]

            # Start HTTP server
            if not self.ipxe_client.is_running():
                self.ipxe_client.start_http_server(http_port)

            # Update deployment status
            deployment["status"] = "running"
            deployment["updated_at"] = int(time.time())

            # Save deployment
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            with open(config_path, "w") as f:
                json.dump(deployment, f, indent=2)

            Logger.info(f"Started deployment {deployment['name']}")
            return True

        except Exception as e:
            Logger.error(f"Failed to start deployment: {str(e)}")
            return False

    def stop_deployment(self, deployment_id: str) -> bool:
        """Stop deployment by stopping the HTTP server.

        Args:
            deployment_id: Deployment ID

        Returns:
            bool: True if successful
        """
        try:
            if deployment_id not in self.deployments:
                Logger.error(f"Deployment not found: {deployment_id}")
                return False

            deployment = self.deployments[deployment_id]

            # Stop HTTP server
            if self.ipxe_client.is_running():
                self.ipxe_client.stop_servers()

            # Update deployment status
            deployment["status"] = "stopped"
            deployment["updated_at"] = int(time.time())

            # Save deployment
            config_path = self.deployment_dir / "configs" / f"{deployment_id}.json"

            with open(config_path, "w") as f:
                json.dump(deployment, f, indent=2)

            Logger.info(f"Stopped deployment {deployment['name']}")
            return True

        except Exception as e:
            Logger.error(f"Failed to stop deployment: {str(e)}")
            return False



from PyQt6.QtWidgets import (QLabel, QGridLayout, QWidget, QVBoxLayout,
                             QComboBox, QPushButton, QTableWidget, QTableWidgetItem,
                             QHeaderView, QHBoxLayout)
from PyQt6.QtCore import Qt

from greenfish.ui.widgets.metrics.base_panel import BaseMetricsPanel
from greenfish.utils.logger import Logger


class TelemetryPanel(BaseMetricsPanel):
    """Panel for displaying telemetry data and metric reports"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.metric_reports = []
        self.report_definitions = []

    def refresh(self):
        """Refresh telemetry data"""
        if not self.client:
            self.show_error("No Redfish client available")
            return

        self.show_loading()
        self.clear_content()

        try:
            # Check if telemetry service is available
            telemetry_service = self.client.get_telemetry_service()
            if not telemetry_service:
                self.show_error("Telemetry service not available on this system")
                return

            # Get metric report definitions
            self.report_definitions = self.client.get_metric_report_definitions()
            if not self.report_definitions:
                self.show_error("No metric report definitions available")
                return

            # Get available metric reports
            self.metric_reports = self.client.get_metric_reports()
            if not self.metric_reports:
                self.show_error("No metric reports available")
                return

            self.hide_status()
            self.display_telemetry_data()

        except Exception as e:
            Logger.error(f"Failed to get telemetry data: {str(e)}")
            self.show_error(str(e))

    def display_telemetry_data(self):
        """Display the telemetry data in the panel"""
        if not self.metric_reports and not self.report_definitions:
            return

        # Create a section for metric report selection
        selection_section, selection_layout = self.create_section("Metric Reports")

        # Create a widget for report selection
        selection_widget = QWidget()
        selection_layout_inner = QHBoxLayout(selection_widget)
        selection_layout_inner.setContentsMargins(10, 5, 10, 5)

        # Create a dropdown for selecting reports
        self.report_combo = QComboBox()
        for report in self.metric_reports:
            if 'Id' in report and 'Name' in report:
                self.report_combo.addItem(report['Name'], report['Id'])

        # Add a refresh button
        refresh_button = QPushButton("View Report")
        refresh_button.clicked.connect(self.load_selected_report)

        selection_layout_inner.addWidget(QLabel("Select Report:"))
        selection_layout_inner.addWidget(self.report_combo, 1)
        selection_layout_inner.addWidget(refresh_button)

        selection_layout.addWidget(selection_widget)
        self.content_layout.insertWidget(0, selection_section)

        # Create a section for the report details
        self.report_section, self.report_layout = self.create_section("Report Details")

        # Add a placeholder message
        placeholder = QLabel("Select a report and click 'View Report' to view details")
        placeholder.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.report_layout.addWidget(placeholder)

        self.content_layout.insertWidget(1, self.report_section)

        # Load the first report if available
        if self.report_combo.count() > 0:
            self.load_selected_report()

    def load_selected_report(self):
        """Load the selected metric report"""
        if self.report_combo.count() == 0:
            return

        # Clear the previous report content
        while self.report_layout.count():
            item = self.report_layout.takeAt(0)
            if item.widget():
                item.widget().deleteLater()

        # Get the selected report ID
        report_id = self.report_combo.currentData()
        if not report_id:
            return

        try:
            # Get the report data
            report_data = self.client.get_metric_report(report_id)
            if not report_data:
                placeholder = QLabel(f"Failed to load report: {report_id}")
                placeholder.setAlignment(Qt.AlignmentFlag.AlignCenter)
                self.report_layout.addWidget(placeholder)
                return

            # Display the report metadata
            metadata_widget = QWidget()
            metadata_layout = QGridLayout(metadata_widget)
            metadata_layout.setContentsMargins(10, 5, 10, 5)

            row = 0

            if 'Id' in report_data:
                metadata_layout.addWidget(QLabel("ID:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['Id']), row, 1)
                row += 1

            if 'Name' in report_data:
                metadata_layout.addWidget(QLabel("Name:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['Name']), row, 1)
                row += 1

            if 'Description' in report_data:
                metadata_layout.addWidget(QLabel("Description:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['Description']), row, 1)
                row += 1

            if 'MetricReportDefinition' in report_data and '@odata.id' in report_data['MetricReportDefinition']:
                metadata_layout.addWidget(QLabel("Definition:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['MetricReportDefinition']['@odata.id'].split('/')[-1]), row, 1)
                row += 1

            if 'ReportingInterval' in report_data:
                metadata_layout.addWidget(QLabel("Reporting Interval:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['ReportingInterval']), row, 1)
                row += 1

            if 'Timestamp' in report_data:
                metadata_layout.addWidget(QLabel("Timestamp:"), row, 0)
                metadata_layout.addWidget(QLabel(report_data['Timestamp']), row, 1)
                row += 1

            self.report_layout.addWidget(metadata_widget)

            # Display the metric values
            if 'MetricValues' in report_data and report_data['MetricValues']:
                # Create a table for the metric values
                table_widget = QTableWidget()
                table_widget.setColumnCount(5)  # MetricId, MetricValue, Timestamp, MetricProperty, Status
                table_widget.setHorizontalHeaderLabels(["Metric ID", "Value", "Timestamp", "Property", "Status"])
                table_widget.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)

                # Add rows for each metric value
                table_widget.setRowCount(len(report_data['MetricValues']))

                for i, metric in enumerate(report_data['MetricValues']):
                    # Metric ID
                    if 'MetricId' in metric:
                        table_widget.setItem(i, 0, QTableWidgetItem(metric['MetricId']))

                    # Metric Value
                    if 'MetricValue' in metric:
                        table_widget.setItem(i, 1, QTableWidgetItem(str(metric['MetricValue'])))

                    # Timestamp
                    if 'Timestamp' in metric:
                        table_widget.setItem(i, 2, QTableWidgetItem(metric['Timestamp']))

                    # Metric Property
                    if 'MetricProperty' in metric:
                        table_widget.setItem(i, 3, QTableWidgetItem(metric['MetricProperty']))

                    # Status
                    if 'Status' in metric:
                        status = metric['Status']
                        health = status.get('Health', 'Unknown')
                        state = status.get('State', 'Unknown')
                        status_text = f"{health} ({state})"
                        status_item = QTableWidgetItem(status_text)

                        # Set color based on health
                        if health == 'OK':
                            status_item.setForeground(Qt.GlobalColor.green)
                        elif health == 'Warning':
                            status_item.setForeground(Qt.GlobalColor.yellow)
                        elif health == 'Critical':
                            status_item.setForeground(Qt.GlobalColor.red)

                        table_widget.setItem(i, 4, status_item)

                self.report_layout.addWidget(table_widget)
            else:
                no_metrics = QLabel("No metric values available in this report")
                no_metrics.setAlignment(Qt.AlignmentFlag.AlignCenter)
                self.report_layout.addWidget(no_metrics)

        except Exception as e:
            Logger.error(f"Failed to load metric report: {str(e)}")
            error_label = QLabel(f"Error loading report: {str(e)}")
            error_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            error_label.setStyleSheet("color: red;")
            self.report_layout.addWidget(error_label)



from PyQt6.QtWidgets import QLabel, QGridLayout, QWidget, QProgressBar
from PyQt6.QtCore import Qt

from greenfish.ui.widgets.metrics.base_panel import BaseMetricsPanel
from greenfish.utils.logger import Logger


class NetworkMetricsPanel(BaseMetricsPanel):
    """Panel for displaying network metrics"""

    def __init__(self, parent=None):
        super().__init__(parent)

    def refresh(self):
        """Refresh network metrics data"""
        if not self.client or not self.resource_id:
            self.show_error("No Redfish client or system ID available")
            return

        self.show_loading()
        self.clear_content()

        try:
            # Get system info from the client
            system_data = self.client.get_system(self.resource_id)
            if not system_data or 'EthernetInterfaces' not in system_data:
                self.show_error("No network data available")
                return

            # Get the ethernet interfaces collection
            interfaces_url = system_data['EthernetInterfaces']['@odata.id']
            interfaces_data = self.client.get_resource(interfaces_url)

            if not interfaces_data or 'Members' not in interfaces_data:
                self.show_error("No network interfaces found")
                return

            # Get detailed information for each interface
            interfaces = []
            for member in interfaces_data['Members']:
                interface_url = member['@odata.id']
                interface_data = self.client.get_resource(interface_url)
                if interface_data:
                    interfaces.append(interface_data)

            if not interfaces:
                self.show_error("No network interface details available")
                return

            self.data = interfaces
            self.hide_status()
            self.display_network_data()

        except Exception as e:
            Logger.error(f"Failed to get network metrics: {str(e)}")
            self.show_error(str(e))

    def display_network_data(self):
        """Display the network interface data in the panel"""
        if not self.data:
            return

        for i, interface in enumerate(self.data):
            # Create a section for each network interface
            interface_name = interface.get('Name', f"Network Interface {i+1}")
            interface_section, section_layout = self.create_section(interface_name)

            interface_widget = QWidget()
            interface_layout = QGridLayout(interface_widget)
            interface_layout.setContentsMargins(10, 5, 10, 5)

            row = 0

            # Basic interface information
            if 'Id' in interface:
                interface_layout.addWidget(QLabel("ID:"), row, 0)
                interface_layout.addWidget(QLabel(interface['Id']), row, 1)
                row += 1

            if 'Description' in interface:
                interface_layout.addWidget(QLabel("Description:"), row, 0)
                interface_layout.addWidget(QLabel(interface['Description']), row, 1)
                row += 1

            # MAC Address
            if 'MACAddress' in interface:
                interface_layout.addWidget(QLabel("MAC Address:"), row, 0)
                interface_layout.addWidget(QLabel(interface['MACAddress']), row, 1)
                row += 1

            # Link status
            if 'LinkStatus' in interface:
                interface_layout.addWidget(QLabel("Link Status:"), row, 0)
                status_label = QLabel(interface['LinkStatus'])

                # Set color based on link status
                if interface['LinkStatus'] == 'LinkUp':
                    status_label.setStyleSheet("color: green;")
                else:
                    status_label.setStyleSheet("color: red;")

                interface_layout.addWidget(status_label, row, 1)
                row += 1

            # Interface speed
            if 'SpeedMbps' in interface:
                interface_layout.addWidget(QLabel("Speed:"), row, 0)
                interface_layout.addWidget(QLabel(f"{interface['SpeedMbps']} Mbps"), row, 1)
                row += 1

            # Auto-negotiation
            if 'AutoNeg' in interface:
                interface_layout.addWidget(QLabel("Auto-Negotiation:"), row, 0)
                interface_layout.addWidget(QLabel("Enabled" if interface['AutoNeg'] else "Disabled"), row, 1)
                row += 1

            # Full duplex
            if 'FullDuplex' in interface:
                interface_layout.addWidget(QLabel("Duplex:"), row, 0)
                interface_layout.addWidget(QLabel("Full" if interface['FullDuplex'] else "Half"), row, 1)
                row += 1

            # MTU size
            if 'MTUSize' in interface:
                interface_layout.addWidget(QLabel("MTU Size:"), row, 0)
                interface_layout.addWidget(QLabel(str(interface['MTUSize'])), row, 1)
                row += 1

            # Status
            if 'Status' in interface:
                status = interface['Status']
                health = status.get('Health', 'Unknown')
                state = status.get('State', 'Unknown')

                interface_layout.addWidget(QLabel("Health:"), row, 0)
                health_label = QLabel(health)

                # Set color based on health
                if health == 'OK':
                    health_label.setStyleSheet("color: green;")
                elif health == 'Warning':
                    health_label.setStyleSheet("color: orange;")
                elif health == 'Critical':
                    health_label.setStyleSheet("color: red;")

                interface_layout.addWidget(health_label, row, 1)
                row += 1

                interface_layout.addWidget(QLabel("State:"), row, 0)
                interface_layout.addWidget(QLabel(state), row, 1)
                row += 1

            # Add a separator
            separator = QWidget()
            separator.setFixedHeight(1)
            separator.setStyleSheet("background-color: #cccccc;")
            interface_layout.addWidget(separator, row, 0, 1, 2)
            row += 1

            # IP configuration
            ip_title = QLabel("IP Configuration")
            ip_title.setStyleSheet("font-weight: bold;")
            interface_layout.addWidget(ip_title, row, 0, 1, 2)
            row += 1

            if 'IPv4Addresses' in interface and interface['IPv4Addresses']:
                for j, ipv4 in enumerate(interface['IPv4Addresses']):
                    if 'Address' in ipv4:
                        interface_layout.addWidget(QLabel(f"IPv4 Address {j+1}:"), row, 0)
                        interface_layout.addWidget(QLabel(ipv4['Address']), row, 1)
                        row += 1

                    if 'SubnetMask' in ipv4:
                        interface_layout.addWidget(QLabel(f"Subnet Mask {j+1}:"), row, 0)
                        interface_layout.addWidget(QLabel(ipv4['SubnetMask']), row, 1)
                        row += 1

                    if 'Gateway' in ipv4:
                        interface_layout.addWidget(QLabel(f"Gateway {j+1}:"), row, 0)
                        interface_layout.addWidget(QLabel(ipv4['Gateway']), row, 1)
                        row += 1

            if 'IPv6Addresses' in interface and interface['IPv6Addresses']:
                for j, ipv6 in enumerate(interface['IPv6Addresses']):
                    if 'Address' in ipv6:
                        interface_layout.addWidget(QLabel(f"IPv6 Address {j+1}:"), row, 0)
                        interface_layout.addWidget(QLabel(ipv6['Address']), row, 1)
                        row += 1

                    if 'PrefixLength' in ipv6:
                        interface_layout.addWidget(QLabel(f"Prefix Length {j+1}:"), row, 0)
                        interface_layout.addWidget(QLabel(str(ipv6['PrefixLength'])), row, 1)
                        row += 1

            # DNS information
            if 'NameServers' in interface and interface['NameServers']:
                # Add a separator
                separator = QWidget()
                separator.setFixedHeight(1)
                separator.setStyleSheet("background-color: #cccccc;")
                interface_layout.addWidget(separator, row, 0, 1, 2)
                row += 1

                dns_title = QLabel("DNS Configuration")
                dns_title.setStyleSheet("font-weight: bold;")
                interface_layout.addWidget(dns_title, row, 0, 1, 2)
                row += 1

                for j, dns in enumerate(interface['NameServers']):
                    interface_layout.addWidget(QLabel(f"DNS Server {j+1}:"), row, 0)
                    interface_layout.addWidget(QLabel(dns), row, 1)
                    row += 1

            # VLAN information
            if 'VLAN' in interface and interface['VLAN']:
                # Add a separator
                separator = QWidget()
                separator.setFixedHeight(1)
                separator.setStyleSheet("background-color: #cccccc;")
                interface_layout.addWidget(separator, row, 0, 1, 2)
                row += 1

                vlan_title = QLabel("VLAN Configuration")
                vlan_title.setStyleSheet("font-weight: bold;")
                interface_layout.addWidget(vlan_title, row, 0, 1, 2)
                row += 1

                vlan = interface['VLAN']

                if 'VLANId' in vlan:
                    interface_layout.addWidget(QLabel("VLAN ID:"), row, 0)
                    interface_layout.addWidget(QLabel(str(vlan['VLANId'])), row, 1)
                    row += 1

                if 'VLANEnable' in vlan:
                    interface_layout.addWidget(QLabel("VLAN Enabled:"), row, 0)
                    interface_layout.addWidget(QLabel("Yes" if vlan['VLANEnable'] else "No"), row, 1)
                    row += 1

            # Interface metrics if available
            if 'Statistics' in interface and interface['Statistics']:
                # Add a separator
                separator = QWidget()
                separator.setFixedHeight(1)
                separator.setStyleSheet("background-color: #cccccc;")
                interface_layout.addWidget(separator, row, 0, 1, 2)
                row += 1

                metrics_title = QLabel("Network Metrics")
                metrics_title.setStyleSheet("font-weight: bold;")
                interface_layout.addWidget(metrics_title, row, 0, 1, 2)
                row += 1

                stats = interface['Statistics']

                if 'RxBytes' in stats:
                    rx_mb = stats['RxBytes'] / (1024 * 1024)
                    interface_layout.addWidget(QLabel("Received:"), row, 0)
                    interface_layout.addWidget(QLabel(f"{rx_mb:.2f} MB"), row, 1)
                    row += 1

                if 'TxBytes' in stats:
                    tx_mb = stats['TxBytes'] / (1024 * 1024)
                    interface_layout.addWidget(QLabel("Transmitted:"), row, 0)
                    interface_layout.addWidget(QLabel(f"{tx_mb:.2f} MB"), row, 1)
                    row += 1

                if 'RxErrors' in stats:
                    interface_layout.addWidget(QLabel("Receive Errors:"), row, 0)
                    error_label = QLabel(str(stats['RxErrors']))

                    # Highlight errors
                    if stats['RxErrors'] > 0:
                        error_label.setStyleSheet("color: red;")

                    interface_layout.addWidget(error_label, row, 1)
                    row += 1

                if 'TxErrors' in stats:
                    interface_layout.addWidget(QLabel("Transmit Errors:"), row, 0)
                    error_label = QLabel(str(stats['TxErrors']))

                    # Highlight errors
                    if stats['TxErrors'] > 0:
                        error_label.setStyleSheet("color: red;")

                    interface_layout.addWidget(error_label, row, 1)
                    row += 1

                if 'RxDropped' in stats:
                    interface_layout.addWidget(QLabel("Dropped Packets (Rx):"), row, 0)
                    interface_layout.addWidget(QLabel(str(stats['RxDropped'])), row, 1)
                    row += 1

                if 'TxDropped' in stats:
                    interface_layout.addWidget(QLabel("Dropped Packets (Tx):"), row, 0)
                    interface_layout.addWidget(QLabel(str(stats['TxDropped'])), row, 1)
                    row += 1

            section_layout.addWidget(interface_widget)
            self.content_layout.insertWidget(i, interface_section)

            # Add separator between interfaces
            if i < len(self.data) - 1:
                separator = QWidget()
                separator.setFixedHeight(2)
                separator.setStyleSheet("background-color: #aaaaaa;")
                self.content_layout.insertWidget(i + 1, separator)



"""Network Configuration Widget."""
from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton,
                           QLabel, QComboBox, QTableWidget, QTableWidgetItem,
                           QHeaderView, QMessageBox, QGroupBox, QTabWidget,
                           QFormLayout, QLineEdit, QCheckBox, QSpinBox,
                           QRadioButton, QButtonGroup, QGridLayout)
from PyQt5.QtCore import Qt, pyqtSignal, QThread

from greenfish.utils.logger import Logger

class NetworkWorker(QThread):
    """Worker thread for network operations."""

    finished = pyqtSignal(bool, str)

    def __init__(self, client, operation, interface=None, config=None):
        """Initialize worker thread.

        Args:
            client: Network client
            operation: Operation to perform
            interface: Network interface
            config: Network configuration
        """
        super().__init__()
        self.client = client
        self.operation = operation
        self.interface = interface
        self.config = config

    def run(self):
        """Run the operation."""
        try:
            result = False
            message = ""

            if self.operation == "apply":
                # Apply network configuration
                result = self.client.apply_network_config(
                    self.interface, self.config
                )

                if result:
                    message = f"Successfully applied network configuration to {self.interface}"
                else:
                    message = f"Failed to apply network configuration to {self.interface}"

            elif self.operation == "reset":
                # Reset network configuration
                result = self.client.reset_network_config(self.interface)

                if result:
                    message = f"Successfully reset network configuration for {self.interface}"
                else:
                    message = f"Failed to reset network configuration for {self.interface}"

            self.finished.emit(result, message)

        except Exception as e:
            Logger.error(f"Error in network operation: {str(e)}")
            self.finished.emit(False, f"Error: {str(e)}")


class NetworkConfigWidget(QWidget):
    """Network Configuration widget."""

    def __init__(self, parent=None):
        """Initialize widget.

        Args:
            parent: Parent widget
        """
        super().__init__(parent)
        self.client = None
        self.interfaces = []
        self.worker = None

        self.init_ui()

    def init_ui(self):
        """Initialize UI components."""
        # Main layout
        main_layout = QVBoxLayout(self)

        # Controls
        controls_layout = QHBoxLayout()

        self.refresh_button = QPushButton("Refresh")
        self.refresh_button.clicked.connect(self.refresh_interfaces)

        self.connection_combo = QComboBox()
        self.connection_combo.addItem("Select Connection Type")
        self.connection_combo.addItem("Redfish")
        self.connection_combo.addItem("IPMI")
        self.connection_combo.currentIndexChanged.connect(self.on_connection_changed)

        controls_layout.addWidget(QLabel("Connection:"))
        controls_layout.addWidget(self.connection_combo)
        controls_layout.addStretch()
        controls_layout.addWidget(self.refresh_button)

        # Tab widget for interfaces
        self.tabs = QTabWidget()

        # Status label
        self.status_label = QLabel()

        # Add widgets to main layout
        main_layout.addLayout(controls_layout)
        main_layout.addWidget(self.tabs)
        main_layout.addWidget(self.status_label)

    def set_client(self, client):
        """Set network client.

        Args:
            client: Network client
        """
        self.client = client
        self.refresh_interfaces()

    def refresh_interfaces(self):
        """Refresh network interfaces."""
        if not self.client or not self.client.is_connected():
            self.status_label.setText("Not connected to a server")
            self.clear_tabs()
            self.interfaces = []
            return

        try:
            # Get network interfaces
            self.interfaces = self.client.get_network_interfaces()

            # Update tabs
            self.update_tabs()

            self.status_label.setText(f"Found {len(self.interfaces)} network interfaces")

        except Exception as e:
            Logger.error(f"Error refreshing network interfaces: {str(e)}")
            self.status_label.setText(f"Error: {str(e)}")

    def clear_tabs(self):
        """Clear all tabs."""
        while self.tabs.count() > 0:
            self.tabs.removeTab(0)

    def update_tabs(self):
        """Update tabs with current interfaces."""
        self.clear_tabs()

        for interface in self.interfaces:
            # Create tab for interface
            interface_widget = QWidget()
            interface_layout = QVBoxLayout(interface_widget)

            # Interface details
            details_group = QGroupBox("Interface Details")
            details_layout = QFormLayout()

            # ID
            id_label = QLabel(interface.get("id", ""))
            details_layout.addRow("ID:", id_label)

            # Name
            name_label = QLabel(interface.get("name", ""))
            details_layout.addRow("Name:", name_label)

            # MAC Address
            mac_label = QLabel(interface.get("mac_address", ""))
            details_layout.addRow("MAC Address:", mac_label)

            # Status
            status_label = QLabel(interface.get("status", ""))
            details_layout.addRow("Status:", status_label)

            # Speed
            speed_label = QLabel(f"{interface.get('speed', 0)} Mbps")
            details_layout.addRow("Speed:", speed_label)

            # MTU
            mtu_label = QLabel(str(interface.get("mtu", 0)))
            details_layout.addRow("MTU:", mtu_label)

            details_group.setLayout(details_layout)

            # IP Configuration
            ip_group = QGroupBox("IP Configuration")
            ip_layout = QVBoxLayout()

            # DHCP/Static selection
            ip_mode_group = QWidget()
            ip_mode_layout = QHBoxLayout(ip_mode_group)
            ip_mode_layout.setContentsMargins(0, 0, 0, 0)

            self.dhcp_radio = QRadioButton("DHCP")
            self.static_radio = QRadioButton("Static IP")

            ip_mode_group_buttons = QButtonGroup(ip_mode_group)
            ip_mode_group_buttons.addButton(self.dhcp_radio)
            ip_mode_group_buttons.addButton(self.static_radio)

            # Set current mode
            if interface.get("dhcp_enabled", True):
                self.dhcp_radio.setChecked(True)
            else:
                self.static_radio.setChecked(True)

            # Connect signals
            self.dhcp_radio.toggled.connect(self.on_ip_mode_changed)
            self.static_radio.toggled.connect(self.on_ip_mode_changed)

            ip_mode_layout.addWidget(self.dhcp_radio)
            ip_mode_layout.addWidget(self.static_radio)
            ip_mode_layout.addStretch()

            ip_layout.addWidget(ip_mode_group)

            # Static IP settings
            static_group = QGroupBox("Static IP Settings")
            static_layout = QFormLayout()

            # IP Address
            self.ip_edit = QLineEdit(interface.get("ip_address", ""))
            static_layout.addRow("IP Address:", self.ip_edit)

            # Subnet Mask
            self.subnet_edit = QLineEdit(interface.get("subnet_mask", ""))
            static_layout.addRow("Subnet Mask:", self.subnet_edit)

            # Gateway
            self.gateway_edit = QLineEdit(interface.get("gateway", ""))
            static_layout.addRow("Gateway:", self.gateway_edit)

            # DNS Servers
            self.dns1_edit = QLineEdit("")
            self.dns2_edit = QLineEdit("")

            # Set DNS servers if available
            dns_servers = interface.get("dns_servers", [])
            if len(dns_servers) > 0:
                self.dns1_edit.setText(dns_servers[0])
            if len(dns_servers) > 1:
                self.dns2_edit.setText(dns_servers[1])

            static_layout.addRow("Primary DNS:", self.dns1_edit)
            static_layout.addRow("Secondary DNS:", self.dns2_edit)

            static_group.setLayout(static_layout)
            ip_layout.addWidget(static_group)

            # Enable/disable static settings based on current mode
            static_group.setEnabled(not interface.get("dhcp_enabled", True))

            # VLAN settings
            vlan_group = QGroupBox("VLAN Settings")
            vlan_layout = QFormLayout()

            # VLAN Enabled
            self.vlan_check = QCheckBox("Enable VLAN")
            self.vlan_check.setChecked(interface.get("vlan_enabled", False))
            self.vlan_check.toggled.connect(self.on_vlan_toggled)

            # VLAN ID
            self.vlan_id_spin = QSpinBox()
            self.vlan_id_spin.setRange(1, 4094)
            self.vlan_id_spin.setValue(interface.get("vlan_id", 1))
            self.vlan_id_spin.setEnabled(interface.get("vlan_enabled", False))

            vlan_layout.addRow("", self.vlan_check)
            vlan_layout.addRow("VLAN ID:", self.vlan_id_spin)

            vlan_group.setLayout(vlan_layout)
            ip_layout.addWidget(vlan_group)

            ip_group.setLayout(ip_layout)

            # Actions
            actions_group = QGroupBox("Actions")
            actions_layout = QHBoxLayout()

            self.apply_button = QPushButton("Apply Settings")
            self.apply_button.clicked.connect(lambda: self.apply_settings(interface))

            self.reset_button = QPushButton("Reset")
            self.reset_button.clicked.connect(lambda: self.reset_settings(interface))

            actions_layout.addWidget(self.apply_button)
            actions_layout.addWidget(self.reset_button)

            actions_group.setLayout(actions_layout)

            # Add groups to interface layout
            interface_layout.addWidget(details_group)
            interface_layout.addWidget(ip_group)
            interface_layout.addWidget(actions_group)
            interface_layout.addStretch()

            # Add tab
            self.tabs.addTab(interface_widget, interface.get("name", f"Interface {interface.get('id', '')}"))

    def on_ip_mode_changed(self, checked):
        """Handle IP mode change.

        Args:
            checked: Whether the radio button is checked
        """
        if not checked:
            return

        # Get current tab widget
        current_widget = self.tabs.currentWidget()
        if not current_widget:
            return

        # Find static group
        static_group = None
        for i in range(current_widget.layout().count()):
            item = current_widget.layout().itemAt(i)
            if item and item.widget() and isinstance(item.widget(), QGroupBox):
                if item.widget().title() == "IP Configuration":
                    ip_group = item.widget()
                    for j in range(ip_group.layout().count()):
                        sub_item = ip_group.layout().itemAt(j)
                        if sub_item and sub_item.widget() and isinstance(sub_item.widget(), QGroupBox):
                            if sub_item.widget().title() == "Static IP Settings":
                                static_group = sub_item.widget()
                                break
                    break

        if static_group:
            # Enable/disable static settings based on mode
            static_group.setEnabled(self.static_radio.isChecked())

    def on_vlan_toggled(self, checked):
        """Handle VLAN toggle.

        Args:
            checked: Whether the checkbox is checked
        """
        # Get current tab widget
        current_widget = self.tabs.currentWidget()
        if not current_widget:
            return

        # Find VLAN ID spinbox
        for i in range(current_widget.layout().count()):
            item = current_widget.layout().itemAt(i)
            if item and item.widget() and isinstance(item.widget(), QGroupBox):
                if item.widget().title() == "IP Configuration":
                    ip_group = item.widget()
                    for j in range(ip_group.layout().count()):
                        sub_item = ip_group.layout().itemAt(j)
                        if sub_item and sub_item.widget() and isinstance(sub_item.widget(), QGroupBox):
                            if sub_item.widget().title() == "VLAN Settings":
                                vlan_group = sub_item.widget()
                                for k in range(vlan_group.layout().rowCount()):
                                    label = vlan_group.layout().itemAt(k, QFormLayout.LabelRole)
                                    field = vlan_group.layout().itemAt(k, QFormLayout.FieldRole)
                                    if label and field and label.widget() and isinstance(label.widget(), QLabel):
                                        if label.widget().text() == "VLAN ID:":
                                            field.widget().setEnabled(checked)
                                            break
                                break
                    break

    def apply_settings(self, interface):
        """Apply network settings.

        Args:
            interface: Network interface
        """
        # Get current tab widget
        current_widget = self.tabs.currentWidget()
        if not current_widget:
            return

        # Build configuration
        config = {}

        # DHCP/Static
        config["dhcp_enabled"] = self.dhcp_radio.isChecked()

        # Static IP settings
        if not config["dhcp_enabled"]:
            config["ip_address"] = self.ip_edit.text()
            config["subnet_mask"] = self.subnet_edit.text()
            config["gateway"] = self.gateway_edit.text()

            # DNS servers
            dns_servers = []
            if self.dns1_edit.text():
                dns_servers.append(self.dns1_edit.text())
            if self.dns2_edit.text():
                dns_servers.append(self.dns2_edit.text())

            config["dns_servers"] = dns_servers

        # VLAN settings
        config["vlan_enabled"] = self.vlan_check.isChecked()
        if config["vlan_enabled"]:
            config["vlan_id"] = self.vlan_id_spin.value()

        # Validate IP address format
        if not config["dhcp_enabled"]:
            import re
            ip_pattern = r"^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$"

            if not re.match(ip_pattern, config["ip_address"]):
                QMessageBox.warning(self, "Invalid IP Address", "Please enter a valid IP address")
                return

            if not re.match(ip_pattern, config["subnet_mask"]):
                QMessageBox.warning(self, "Invalid Subnet Mask", "Please enter a valid subnet mask")
                return

            if config["gateway"] and not re.match(ip_pattern, config["gateway"]):
                QMessageBox.warning(self, "Invalid Gateway", "Please enter a valid gateway address")
                return

            for dns in config["dns_servers"]:
                if not re.match(ip_pattern, dns):
                    QMessageBox.warning(self, "Invalid DNS Server", "Please enter valid DNS server addresses")
                    return

        # Confirm changes
        reply = QMessageBox.question(
            self, "Apply Network Settings",
            f"Are you sure you want to apply these network settings to {interface.get('name', '')}?\n\n"
            "This may cause temporary network disruption.",
            QMessageBox.Yes | QMessageBox.No, QMessageBox.No
        )

        if reply == QMessageBox.Yes:
            # Disable UI during operation
            self.setEnabled(False)
            self.status_label.setText(f"Applying network settings to {interface.get('name', '')}...")

            # Start worker thread
            self.worker = NetworkWorker(
                self.client, "apply", interface.get("id", ""), config
            )
            self.worker.finished.connect(self.on_operation_finished)
            self.worker.start()

    def reset_settings(self, interface):
        """Reset network settings.

        Args:
            interface: Network interface
        """
        # Confirm reset
        reply = QMessageBox.question(
            self, "Reset Network Settings",
            f"Are you sure you want to reset the network settings for {interface.get('name', '')}?\n\n"
            "This will restore the default configuration and may cause temporary network disruption.",
            QMessageBox.Yes | QMessageBox.No, QMessageBox.No
        )

        if reply == QMessageBox.Yes:
            # Disable UI during operation
            self.setEnabled(False)
            self.status_label.setText(f"Resetting network settings for {interface.get('name', '')}...")

            # Start worker thread
            self.worker = NetworkWorker(
                self.client, "reset", interface.get("id", "")
            )
            self.worker.finished.connect(self.on_operation_finished)
            self.worker.start()

    def on_operation_finished(self, success, message):
        """Handle operation completion.

        Args:
            success: Whether operation was successful
            message: Result message
        """
        # Re-enable UI
        self.setEnabled(True)

        # Show result
        self.status_label.setText(message)

        if success:
            # Refresh interfaces
            self.refresh_interfaces()
        else:
            # Show error message
            QMessageBox.critical(self, "Operation Failed", message)

    def on_connection_changed(self, index):
        """Handle connection type change.

        Args:
            index: Selected index
        """
        if index == 0:
            # No connection selected
            self.client = None
            self.clear_tabs()
            self.interfaces = []
            self.status_label.setText("No connection selected")
        elif index == 1:
            # Redfish connection
            if hasattr(self.parent(), "get_redfish_client"):
                redfish_client = self.parent().get_redfish_client()
                if redfish_client and redfish_client.is_connected():
                    from greenfish.core.network import NetworkClient
                    self.client = NetworkClient()
                    self.client.connect_redfish(redfish_client)
                    self.refresh_interfaces()
                else:
                    self.status_label.setText("Redfish client not connected")
            else:
                self.status_label.setText("Redfish client not available")
        elif index == 2:
            # IPMI connection
            if hasattr(self.parent(), "get_ipmi_client"):
                ipmi_client = self.parent().get_ipmi_client()
                if ipmi_client and ipmi_client.is_connected():
                    from greenfish.core.network import NetworkClient
                    self.client = NetworkClient()
                    self.client.connect_ipmi(ipmi_client)
                    self.refresh_interfaces()
                else:
                    self.status_label.setText("IPMI client not connected")
            else:
                self.status_label.setText("IPMI client not available")



"""IPMI 2.0 protocol implementation."""
import socket
import struct
import random
import time
import hmac
import hashlib
from typing import Dict, Any, List, Optional, Tuple, Union, Callable

from greenfish.utils.logger import Logger

# IPMI constants
IPMI_VERSION_2_0 = 0x20
RMCP_PORT = 623
RMCP_HEADER_SIZE = 4

# IPMI message types
IPMI_NETFN_APP = 0x06
IPMI_CMD_GET_DEVICE_ID = 0x01
IPMI_CMD_GET_CHANNEL_AUTH_CAP = 0x38
IPMI_CMD_GET_SESSION_CHALLENGE = 0x39
IPMI_CMD_ACTIVATE_SESSION = 0x3A
IPMI_CMD_SET_SESSION_PRIVILEGE = 0x3B
IPMI_CMD_CLOSE_SESSION = 0x3C

# Authentication types
IPMI_AUTH_NONE = 0x00
IPMI_AUTH_MD2 = 0x01
IPMI_AUTH_MD5 = 0x02
IPMI_AUTH_PASSWORD = 0x04
IPMI_AUTH_OEM = 0x05
IPMI_AUTH_RMCP_PLUS = 0x06

# Privilege levels
IPMI_PRIVILEGE_CALLBACK = 0x01
IPMI_PRIVILEGE_USER = 0x02
IPMI_PRIVILEGE_OPERATOR = 0x03
IPMI_PRIVILEGE_ADMINISTRATOR = 0x04
IPMI_PRIVILEGE_OEM = 0x05

class IPMIProtocol:
    """IPMI 2.0 protocol handler."""

    def __init__(self, host: str, port: int = RMCP_PORT, username: str = "", password: str = ""):
        """Initialize IPMI protocol handler.

        Args:
            host: BMC hostname or IP address
            port: IPMI port (default: 623)
            username: Username for authentication
            password: Password for authentication
        """
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.socket = None
        self.session_id = 0
        self.sequence_number = 0
        self.auth_type = IPMI_AUTH_MD5  # Default to MD5 authentication
        self.privilege_level = IPMI_PRIVILEGE_ADMINISTRATOR
        self.session_active = False
        self.timeout = 5.0  # Socket timeout in seconds

    def connect(self) -> bool:
        """Establish connection to BMC.

        Returns:
            bool: True if connection successful, False otherwise
        """
        try:
            # Create socket
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self.socket.settimeout(self.timeout)

            # Get channel authentication capabilities
            auth_cap = self._get_channel_auth_capabilities()
            if not auth_cap:
                Logger.error("Failed to get channel authentication capabilities")
                return False

            # Get session challenge
            challenge = self._get_session_challenge()
            if not challenge:
                Logger.error("Failed to get session challenge")
                return False

            # Activate session
            if not self._activate_session(challenge):
                Logger.error("Failed to activate session")
                return False

            # Set session privilege level
            if not self._set_session_privilege():
                Logger.error("Failed to set session privilege level")
                return False

            self.session_active = True
            return True

        except Exception as e:
            Logger.error(f"IPMI connection error: {str(e)}")
            if self.socket:
                self.socket.close()
                self.socket = None
            return False

    def disconnect(self) -> None:
        """Close connection to BMC."""
        if self.session_active:
            try:
                self._close_session()
            except Exception as e:
                Logger.error(f"Error closing IPMI session: {str(e)}")

        if self.socket:
            self.socket.close()
            self.socket = None

        self.session_active = False

    def send_command(self, netfn: int, command: int, data: bytes = b'') -> Dict[str, Any]:
        """Send IPMI command and return response.

        Args:
            netfn: Network function code
            command: Command code
            data: Command data bytes

        Returns:
            Dict containing completion code and response data
        """
        if not self.socket or not self.session_active:
            raise RuntimeError("Not connected to BMC")

        # Build IPMI message
        msg = self._build_ipmi_message(netfn, command, data)

        # Send message
        self.socket.sendto(msg, (self.host, self.port))

        # Receive response
        response, _ = self.socket.recvfrom(1024)

        # Parse response
        return self._parse_ipmi_response(response, netfn, command)

    def get_device_id(self) -> Dict[str, Any]:
        """Get BMC device ID information.

        Returns:
            Dict containing BMC device information
        """
        response = self.send_command(IPMI_NETFN_APP, IPMI_CMD_GET_DEVICE_ID)
        if response['completion_code'] != 0:
            raise RuntimeError(f"Failed to get device ID: {response['completion_code']:#x}")

        data = response['data']
        if len(data) < 12:
            raise RuntimeError("Invalid device ID response")

        # Parse device ID response
        device_id = data[0]
        device_revision = data[1] & 0x0F
        firmware_revision_1 = data[2]
        firmware_revision_2 = data[3]
        ipmi_version = data[4]
        manufacturer_id = (data[7] << 16) | (data[6] << 8) | data[5]
        product_id = (data[9] << 8) | data[8]

        return {
            'device_id': device_id,
            'device_revision': device_revision,
            'firmware_revision': f"{firmware_revision_1}.{firmware_revision_2:02x}",
            'ipmi_version': f"{ipmi_version >> 4}.{ipmi_version & 0x0F}",
            'manufacturer_id': manufacturer_id,
            'product_id': product_id
        }

    def _get_channel_auth_capabilities(self) -> Dict[str, Any]:
        """Get channel authentication capabilities.

        Returns:
            Dict containing authentication capabilities
        """
        # Build request data
        data = bytes([
            0x0E,  # Channel number (E = current channel)
            self.privilege_level  # Requested privilege level
        ])

        # Send request
        response = self._send_rmcp_message(IPMI_NETFN_APP, IPMI_CMD_GET_CHANNEL_AUTH_CAP, data)
        if response['completion_code'] != 0:
            Logger.error(f"Get channel auth capabilities failed: {response['completion_code']:#x}")
            return {}

        # Parse response
        data = response['data']
        if len(data) < 8:
            Logger.error("Invalid channel auth capabilities response")
            return {}

        channel_number = data[0]
        auth_type_support = data[1]

        return {
            'channel_number': channel_number,
            'auth_type_support': auth_type_support,
            'auth_types': {
                'none': bool(auth_type_support & (1 << IPMI_AUTH_NONE)),
                'md2': bool(auth_type_support & (1 << IPMI_AUTH_MD2)),
                'md5': bool(auth_type_support & (1 << IPMI_AUTH_MD5)),
                'password': bool(auth_type_support & (1 << IPMI_AUTH_PASSWORD)),
                'oem': bool(auth_type_support & (1 << IPMI_AUTH_OEM)),
                'rmcp_plus': bool(auth_type_support & (1 << IPMI_AUTH_RMCP_PLUS))
            }
        }

    def _get_session_challenge(self) -> bytes:
        """Get session challenge.

        Returns:
            Challenge bytes or empty bytes on failure
        """
        # Build request data
        data = bytes([
            self.auth_type  # Authentication type
        ])

        # Add username padded to 16 bytes
        username_bytes = self.username.encode('ascii')
        username_bytes = username_bytes[:16].ljust(16, b'\0')
        data += username_bytes

        # Send request
        response = self._send_rmcp_message(IPMI_NETFN_APP, IPMI_CMD_GET_SESSION_CHALLENGE, data)
        if response['completion_code'] != 0:
            Logger.error(f"Get session challenge failed: {response['completion_code']:#x}")
            return b''

        # Parse response
        data = response['data']
        if len(data) < 20:
            Logger.error("Invalid session challenge response")
            return b''

        self.session_id = struct.unpack("<I", data[0:4])[0]
        challenge = data[4:20]

        return challenge

    def _activate_session(self, challenge: bytes) -> bool:
        """Activate session using challenge.

        Args:
            challenge: Challenge bytes from get_session_challenge

        Returns:
            bool: True if session activated successfully
        """
        # Generate random initial sequence number
        self.sequence_number = random.randint(0, 0xFFFFFFFF)

        # Calculate authentication code
        auth_code = self._calculate_auth_code(challenge)

        # Build request data
        data = bytes([
            self.auth_type,  # Authentication type
            self.privilege_level  # Requested privilege level
        ])

        # Add challenge string
        data += challenge

        # Add initial outbound sequence number
        data += struct.pack("<I", self.sequence_number)

        # Send request
        response = self._send_rmcp_message(IPMI_NETFN_APP, IPMI_CMD_ACTIVATE_SESSION, data, auth_code)
        if response['completion_code'] != 0:
            Logger.error(f"Activate session failed: {response['completion_code']:#x}")
            return False

        # Parse response
        data = response['data']
        if len(data) < 10:
            Logger.error("Invalid activate session response")
            return False

        # Update session ID and sequence number
        self.session_id = struct.unpack("<I", data[1:5])[0]
        self.sequence_number = struct.unpack("<I", data[5:9])[0]

        return True

    def _set_session_privilege(self) -> bool:
        """Set session privilege level.

        Returns:
            bool: True if privilege level set successfully
        """
        # Build request data
        data = bytes([
            0x00,  # Reserved
            self.privilege_level  # Requested privilege level
        ])

        # Send request
        response = self.send_command(IPMI_NETFN_APP, IPMI_CMD_SET_SESSION_PRIVILEGE, data)
        if response['completion_code'] != 0:
            Logger.error(f"Set session privilege failed: {response['completion_code']:#x}")
            return False

        # Parse response
        data = response['data']
        if len(data) < 1:
            Logger.error("Invalid set session privilege response")
            return False

        # Check if requested privilege level was granted
        granted_level = data[0]
        if granted_level != self.privilege_level:
            Logger.warning(f"Requested privilege level {self.privilege_level} not granted, got {granted_level}")

        return True

    def _close_session(self) -> bool:
        """Close the current session.

        Returns:
            bool: True if session closed successfully
        """
        # Build request data with session ID
        data = struct.pack("<I", self.session_id)

        # Send request
        response = self.send_command(IPMI_NETFN_APP, IPMI_CMD_CLOSE_SESSION, data)

        # Check completion code
        return response['completion_code'] == 0

    def _send_rmcp_message(self, netfn: int, command: int, data: bytes, auth_code: bytes = None) -> Dict[str, Any]:
        """Send RMCP message without session.

        Args:
            netfn: Network function code
            command: Command code
            data: Command data bytes
            auth_code: Authentication code (optional)

        Returns:
            Dict containing completion code and response data
        """
        if not self.socket:
            raise RuntimeError("Socket not initialized")

        # Build RMCP header
        rmcp_header = bytes([
            0x06,  # RMCP Version 1.0
            0x00,  # Reserved
            0x00,  # Reserved
            0x07   # RMCP message class (IPMI)
        ])

        # Build IPMI header
        ipmi_header = bytes([
            0x00,  # Authentication type (none)
            0x00, 0x00, 0x00, 0x00,  # Session ID (0 for pre-session commands)
            0x00, 0x00, 0x00, 0x00,  # Sequence number (0 for pre-session commands)
            (netfn << 2) | 0x00,  # Network function and LUN
            0x00,  # Checksum
            0x20,  # Requester address (BMC)
            0x20,  # Responder address (BMC)
            command  # Command code
        ])

        # Calculate header checksum
        header_checksum = (-sum(ipmi_header[9:12])) & 0xFF
        ipmi_header = ipmi_header[:10] + bytes([header_checksum]) + ipmi_header[11:]

        # Calculate data checksum
        data_checksum = (-sum(ipmi_header[11:] + data)) & 0xFF

        # Build complete message
        message = rmcp_header + ipmi_header + data + bytes([data_checksum])

        # Send message
        self.socket.sendto(message, (self.host, self.port))

        # Receive response
        response, _ = self.socket.recvfrom(1024)

        # Parse response
        return self._parse_rmcp_response(response)

    def _build_ipmi_message(self, netfn: int, command: int, data: bytes) -> bytes:
        """Build IPMI message with session information.

        Args:
            netfn: Network function code
            command: Command code
            data: Command data bytes

        Returns:
            Complete IPMI message bytes
        """
        # Build RMCP header
        rmcp_header = bytes([
            0x06,  # RMCP Version 1.0
            0x00,  # Reserved
            0x00,  # Reserved
            0x07   # RMCP message class (IPMI)
        ])

        # Increment sequence number
        self.sequence_number = (self.sequence_number + 1) & 0xFFFFFFFF

        # Build IPMI header
        ipmi_header = bytes([
            self.auth_type,  # Authentication type
        ])

        # Add session ID
        ipmi_header += struct.pack("<I", self.session_id)

        # Add sequence number
        ipmi_header += struct.pack("<I", self.sequence_number)

        # Calculate authentication code if needed
        auth_code = b'\x00' * 16  # Placeholder for auth code
        if self.auth_type != IPMI_AUTH_NONE:
            # In a real implementation, calculate the auth code here
            pass

        # Add authentication code
        ipmi_header += auth_code

        # Add message header
        ipmi_header += bytes([
            (netfn << 2) | 0x00,  # Network function and LUN
            0x00,  # Checksum (placeholder)
            0x20,  # Requester address (BMC)
            0x20,  # Responder address (BMC)
            command  # Command code
        ])

        # Calculate header checksum
        header_checksum = (-sum(ipmi_header[-5:-2])) & 0xFF
        ipmi_header = ipmi_header[:-4] + bytes([header_checksum]) + ipmi_header[-3:]

        # Calculate data checksum
        data_checksum = (-sum(ipmi_header[-3:] + data)) & 0xFF

        # Build complete message
        return rmcp_header + ipmi_header + data + bytes([data_checksum])

    def _parse_rmcp_response(self, response: bytes) -> Dict[str, Any]:
        """Parse RMCP response.

        Args:
            response: Response bytes

        Returns:
            Dict containing completion code and response data
        """
        if len(response) < RMCP_HEADER_SIZE + 10:
            raise RuntimeError("Invalid RMCP response")

        # Skip RMCP header
        ipmi_msg = response[RMCP_HEADER_SIZE:]

        # Parse IPMI message
        if ipmi_msg[0] != 0x00:  # Auth type
            Logger.warning(f"Unexpected authentication type: {ipmi_msg[0]:#x}")

        # Find start of IPMI response data
        data_start = 9  # Skip auth type, session ID, sequence number

        # Extract completion code
        completion_code = ipmi_msg[data_start + 4]

        # Extract response data
        data = ipmi_msg[data_start + 5:-1]  # Exclude checksum

        return {
            'completion_code': completion_code,
            'data': data
        }

    def _parse_ipmi_response(self, response: bytes, netfn: int, command: int) -> Dict[str, Any]:
        """Parse IPMI response with session information.

        Args:
            response: Response bytes
            netfn: Expected network function code
            command: Expected command code

        Returns:
            Dict containing completion code and response data
        """
        if len(response) < RMCP_HEADER_SIZE + 21:
            raise RuntimeError("Invalid IPMI response")

        # Skip RMCP header
        ipmi_msg = response[RMCP_HEADER_SIZE:]

        # Verify authentication type
        if ipmi_msg[0] != self.auth_type:
            Logger.warning(f"Authentication type mismatch: expected {self.auth_type:#x}, got {ipmi_msg[0]:#x}")

        # Verify session ID
        session_id = struct.unpack("<I", ipmi_msg[1:5])[0]
        if session_id != self.session_id:
            Logger.warning(f"Session ID mismatch: expected {self.session_id:#x}, got {session_id:#x}")

        # Skip auth type, session ID, sequence number, auth code
        data_start = 21

        # Verify network function
        resp_netfn = (ipmi_msg[data_start] >> 2) & 0x3F
        expected_resp_netfn = (netfn | 0x01)  # Response netfn is request netfn + 1
        if resp_netfn != expected_resp_netfn:
            Logger.warning(f"Network function mismatch: expected {expected_resp_netfn:#x}, got {resp_netfn:#x}")

        # Verify command
        resp_cmd = ipmi_msg[data_start + 3]
        if resp_cmd != command:
            Logger.warning(f"Command mismatch: expected {command:#x}, got {resp_cmd:#x}")

        # Extract completion code
        completion_code = ipmi_msg[data_start + 4]

        # Extract response data
        data = ipmi_msg[data_start + 5:-1]  # Exclude checksum

        return {
            'completion_code': completion_code,
            'data': data
        }

    def _calculate_auth_code(self, challenge: bytes) -> bytes:
        """Calculate authentication code.

        Args:
            challenge: Challenge bytes

        Returns:
            Authentication code bytes
        """
        if self.auth_type == IPMI_AUTH_NONE:
            return b'\x00' * 16

        elif self.auth_type == IPMI_AUTH_MD5:
            # MD5 hash of password + session ID + challenge + password
            password_bytes = self.password.encode('ascii')
            session_id_bytes = struct.pack("<I", self.session_id)

            auth_data = password_bytes + session_id_bytes + challenge + password_bytes
            return hashlib.md5(auth_data).digest()

        else:
            Logger.warning(f"Unsupported authentication type: {self.auth_type}")
            return b'\x00' * 16



"""IPMI user operations module."""
from typing import Dict, Any, List, Optional

from greenfish.utils.logger import Logger

# IPMI constants
IPMI_NETFN_APP = 0x06
IPMI_CMD_GET_USER_ACCESS = 0x44
IPMI_CMD_GET_USER_NAME = 0x45
IPMI_CMD_SET_USER_NAME = 0x46
IPMI_CMD_SET_USER_PASSWORD = 0x47
IPMI_CMD_SET_USER_ACCESS = 0x43
IPMI_CMD_GET_CHANNEL_ACCESS = 0x41
IPMI_CMD_SET_CHANNEL_ACCESS = 0x40

# User privilege levels
IPMI_PRIVILEGE_CALLBACK = 0x01
IPMI_PRIVILEGE_USER = 0x02
IPMI_PRIVILEGE_OPERATOR = 0x03
IPMI_PRIVILEGE_ADMINISTRATOR = 0x04
IPMI_PRIVILEGE_OEM = 0x05

# Password operations
IPMI_PASSWORD_SET_PASSWORD = 0x00
IPMI_PASSWORD_TEST_PASSWORD = 0x01
IPMI_PASSWORD_DISABLE_USER = 0x02
IPMI_PASSWORD_ENABLE_USER = 0x03

# Password types
IPMI_PASSWORD_TYPE_16_BYTES = 0x00
IPMI_PASSWORD_TYPE_20_BYTES = 0x01

class UserOperations:
    """IPMI user operations handler."""

    def __init__(self, protocol):
        """Initialize user operations handler.

        Args:
            protocol: IPMI protocol handler
        """
        self.protocol = protocol

    def get_user_access(self, channel: int = 1, user_id: int = 1) -> Dict[str, Any]:
        """Get user access information.

        Args:
            channel: Channel number
            user_id: User ID

        Returns:
            Dict containing user access information
        """
        request_data = bytes([
            channel & 0x0F,
            user_id
        ])

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_GET_USER_ACCESS, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to get user access: {response['completion_code']:#x}")
            return {}

        data = response['data']
        if len(data) < 4:
            Logger.error("Invalid user access response")
            return {}

        return {
            'max_user_ids': data[0] & 0x3F,
            'enabled_user_ids': data[1] & 0x3F,
            'fixed_user_ids': data[2] & 0x3F,
            'user_access': {
                'callback_allowed': bool(data[3] & 0x01),
                'link_auth_allowed': bool(data[3] & 0x02),
                'ipmi_msg_allowed': bool(data[3] & 0x04),
                'privilege_level': data[3] >> 4
            }
        }

    def get_user_name(self, user_id: int) -> str:
        """Get user name.

        Args:
            user_id: User ID

        Returns:
            User name
        """
        request_data = bytes([user_id])

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_GET_USER_NAME, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to get user name: {response['completion_code']:#x}")
            return ""

        data = response['data']
        if not data:
            return ""

        # Convert bytes to string, strip null bytes
        try:
            name = data.decode('ascii').rstrip('\x00')
        except UnicodeDecodeError:
            name = data.hex()

        return name

    def set_user_name(self, user_id: int, name: str) -> bool:
        """Set user name.

        Args:
            user_id: User ID
            name: User name (max 16 bytes)

        Returns:
            bool: True if successful
        """
        # Encode name and pad to 16 bytes
        name_bytes = name.encode('ascii')[:16].ljust(16, b'\x00')

        request_data = bytes([user_id]) + name_bytes

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_SET_USER_NAME, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to set user name: {response['completion_code']:#x}")
            return False

        return True

    def set_user_password(self, user_id: int, password: str, operation: int = IPMI_PASSWORD_SET_PASSWORD,
                         password_type: int = IPMI_PASSWORD_TYPE_16_BYTES) -> bool:
        """Set user password.

        Args:
            user_id: User ID
            password: Password
            operation: Password operation
            password_type: Password type

        Returns:
            bool: True if successful
        """
        # Encode password and pad appropriately
        if password_type == IPMI_PASSWORD_TYPE_16_BYTES:
            password_bytes = password.encode('ascii')[:16].ljust(16, b'\x00')
        else:  # IPMI_PASSWORD_TYPE_20_BYTES
            password_bytes = password.encode('ascii')[:20].ljust(20, b'\x00')

        request_data = bytes([
            user_id,
            (operation & 0x03) | ((password_type & 0x01) << 4)
        ]) + password_bytes

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_SET_USER_PASSWORD, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to set user password: {response['completion_code']:#x}")
            return False

        return True

    def enable_user(self, user_id: int) -> bool:
        """Enable user.

        Args:
            user_id: User ID

        Returns:
            bool: True if successful
        """
        return self.set_user_password(user_id, "", IPMI_PASSWORD_ENABLE_USER)

    def disable_user(self, user_id: int) -> bool:
        """Disable user.

        Args:
            user_id: User ID

        Returns:
            bool: True if successful
        """
        return self.set_user_password(user_id, "", IPMI_PASSWORD_DISABLE_USER)

    def test_password(self, user_id: int, password: str) -> bool:
        """Test if password is correct.

        Args:
            user_id: User ID
            password: Password to test

        Returns:
            bool: True if password is correct
        """
        result = self.set_user_password(user_id, password, IPMI_PASSWORD_TEST_PASSWORD)
        return result

    def set_user_access(self, user_id: int, channel: int = 1, privilege: int = IPMI_PRIVILEGE_USER,
                       callback: bool = False, link_auth: bool = True, ipmi_msg: bool = True,
                       enable_user: bool = True) -> bool:
        """Set user access.

        Args:
            user_id: User ID
            channel: Channel number
            privilege: Privilege level
            callback: Enable callback
            link_auth: Enable link authentication
            ipmi_msg: Enable IPMI messaging
            enable_user: Enable user

        Returns:
            bool: True if successful
        """
        # Build access flags
        access = 0
        if callback:
            access |= 0x01
        if link_auth:
            access |= 0x02
        if ipmi_msg:
            access |= 0x04
        if enable_user:
            access |= 0x80

        # Build request data
        request_data = bytes([
            (channel & 0x0F) | 0x80,  # Channel number with change bit set
            (user_id & 0x3F) | (access & 0xC0),  # User ID with enable/disable bits
            (access & 0x3F) | ((privilege & 0x0F) << 4),  # Access flags with privilege
            0x00  # Reserved
        ])

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_SET_USER_ACCESS, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to set user access: {response['completion_code']:#x}")
            return False

        return True

    def get_channel_access(self, channel: int = 1, get_non_volatile: bool = False) -> Dict[str, Any]:
        """Get channel access information.

        Args:
            channel: Channel number
            get_non_volatile: Get non-volatile settings

        Returns:
            Dict containing channel access information
        """
        request_data = bytes([
            channel & 0x0F,
            0x01 if get_non_volatile else 0x00
        ])

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_GET_CHANNEL_ACCESS, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to get channel access: {response['completion_code']:#x}")
            return {}

        data = response['data']
        if len(data) < 2:
            Logger.error("Invalid channel access response")
            return {}

        return {
            'access_mode': data[0] & 0x07,
            'user_level_auth': bool(data[0] & 0x10),
            'per_message_auth': bool(data[0] & 0x20),
            'pef_alerting': bool(data[0] & 0x40),
            'channel_privilege': data[1] & 0x0F
        }

    def set_channel_access(self, channel: int = 1, access_mode: int = None, user_level_auth: bool = None,
                          per_message_auth: bool = None, pef_alerting: bool = None,
                          privilege: int = None, volatile: bool = True) -> bool:
        """Set channel access.

        Args:
            channel: Channel number
            access_mode: Access mode
            user_level_auth: Enable user level authentication
            per_message_auth: Enable per-message authentication
            pef_alerting: Enable PEF alerting
            privilege: Channel privilege level
            volatile: Set volatile settings

        Returns:
            bool: True if successful
        """
        # Build access flags
        access_byte = 0
        access_mask = 0

        if access_mode is not None:
            access_byte |= access_mode & 0x07
            access_mask |= 0x07

        if user_level_auth is not None:
            if user_level_auth:
                access_byte |= 0x10
            access_mask |= 0x10

        if per_message_auth is not None:
            if per_message_auth:
                access_byte |= 0x20
            access_mask |= 0x20

        if pef_alerting is not None:
            if pef_alerting:
                access_byte |= 0x40
            access_mask |= 0x40

        # Build privilege byte
        priv_byte = 0
        priv_mask = 0

        if privilege is not None:
            priv_byte |= privilege & 0x0F
            priv_mask |= 0x0F

        # Set change bits
        if access_mask:
            access_byte |= 0x80

        if priv_mask:
            priv_byte |= 0x80

        # Build request data
        request_data = bytes([
            channel & 0x0F,
            (0x01 if volatile else 0x02),  # 0x01 = volatile, 0x02 = non-volatile
            access_byte,
            priv_byte
        ])

        response = self.protocol.send_command(IPMI_NETFN_APP, IPMI_CMD_SET_CHANNEL_ACCESS, request_data)

        if response['completion_code'] != 0:
            Logger.error(f"Failed to set channel access: {response['completion_code']:#x}")
            return False

        return True

    def list_users(self, max_users: int = 16) -> List[Dict[str, Any]]:
        """List all users.

        Args:
            max_users: Maximum number of users to check

        Returns:
            List of user dictionaries
        """
        users = []

        for user_id in range(1, max_users + 1):
            # Get user access
            access = self.get_user_access(1, user_id)
            if not access:
                continue

            # Get user name
            name = self.get_user_name(user_id)
            if not name:
                continue

            # Create user entry
            user = {
                'id': user_id,
                'name': name,
                'privilege': access.get('user_access', {}).get('privilege_level', 0),
                'enabled': access.get('user_access', {}).get('ipmi_msg_allowed', False)
            }

            users.append(user)

        return users

    def create_user(self, name: str, password: str, privilege: int = IPMI_PRIVILEGE_USER,
                   channel: int = 1) -> bool:
        """Create a new user.

        Args:
            name: User name
            password: User password
            privilege: Privilege level
            channel: Channel number

        Returns:
            bool: True if successful
        """
        # Find first available user ID
        max_users = 16  # Default max users
        for user_id in range(1, max_users + 1):
            existing_name = self.get_user_name(user_id)
            if not existing_name:
                # Found an empty slot
                break
        else:
            Logger.error("No available user slots")
            return False

        # Set user name
        if not self.set_user_name(user_id, name):
            return False

        # Set user password
        if not self.set_user_password(user_id, password):
            return False

        # Set user access
        if not self.set_user_access(user_id, channel, privilege):
            return False

        # Enable user
        if not self.enable_user(user_id):
            return False

        return True

    def delete_user(self, user_id: int) -> bool:
        """Delete a user.

        Args:
            user_id: User ID

        Returns:
            bool: True if successful
        """
        # Disable user
        if not self.disable_user(user_id):
            return False

        # Clear user name
        if not self.set_user_name(user_id, ""):
            return False

        return True



"""Bare Metal Provisioning module for Redfish Desktop."""
from greenfish.core.provisioning.ipxe import IPXEClient
from greenfish.core.provisioning.deployment import OSDeployment
from greenfish.core.provisioning.network import NetworkConfiguration

__all__ = ['IPXEClient', 'OSDeployment', 'NetworkConfiguration']



"""IPMI protocol implementation."""
from typing import Dict, Any, Optional, List, Union, Tuple
import subprocess
import re
import os
import tempfile
import time
import platform
from greenfish.utils.logger import Logger

class IPMIProtocol:
    """IPMI 2.0 protocol implementation using ipmitool."""

    def __init__(self):
        """Initialize IPMI protocol handler."""
        self.host = None
        self.port = None
        self.username = None
        self.password = None
        self.interface = None
        self.timeout = None
        self.kg_key = None
        self.privilege_level = None
        self.connected = False
        self._check_ipmitool()

    def _check_ipmitool(self) -> bool:
        """Check if ipmitool is available.

        Returns:
            bool: True if ipmitool is available
        """
        try:
            result = subprocess.run(
                ["ipmitool", "-V"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                timeout=5
            )

            if result.returncode == 0:
                Logger.debug(f"ipmitool found: {result.stdout.strip()}")
                return True
            else:
                Logger.error(f"ipmitool check failed: {result.stderr.strip()}")
                return False

        except FileNotFoundError:
            Logger.error("ipmitool not found in PATH")
            return False
        except Exception as e:
            Logger.error(f"Error checking for ipmitool: {str(e)}")
            return False

    def connect(self, host: str, username: str, password: str, port: int = 623,
                interface: str = 'lanplus', timeout: int = 5,
                kg_key: str = None, privilege_level: str = 'ADMINISTRATOR') -> bool:
        """Connect to a BMC via IPMI.

        Args:
            host: BMC hostname or IP address
            username: BMC username
            password: BMC password
            port: IPMI port (default: 623)
            interface: IPMI interface (default: lanplus)
            timeout: Connection timeout in seconds (default: 5)
            kg_key: K_g key for authentication (default: None)
            privilege_level: IPMI privilege level (default: ADMINISTRATOR)

        Returns:
            bool: True if connection successful
        """
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.interface = interface
        self.timeout = timeout
        self.kg_key = kg_key
        self.privilege_level = privilege_level

        # Test connection by getting device ID
        try:
            result = self.send_command(0x06, 0x01)  # NetFn App, Cmd Get Device ID
            if result['completion_code'] == 0:
                self.connected = True
                return True
            else:
                Logger.error(f"IPMI connection test failed: {result['completion_code']:#x}")
                self.connected = False
                return False

        except Exception as e:
            Logger.error(f"IPMI connection error: {str(e)}")
            self.connected = False
            return False

    def disconnect(self) -> bool:
        """Disconnect from BMC.

        Returns:
            bool: True if disconnection successful
        """
        # No actual disconnection needed for ipmitool
        self.connected = False
        return True

    def _build_ipmitool_base_cmd(self) -> List[str]:
        """Build base ipmitool command with connection parameters.

        Returns:
            List[str]: Base command arguments
        """
        cmd = ["ipmitool"]

        # Add interface
        cmd.extend(["-I", self.interface])

        # Add connection details
        cmd.extend(["-H", self.host])
        cmd.extend(["-p", str(self.port)])
        cmd.extend(["-U", self.username])
        cmd.extend(["-P", self.password])

        # Add timeout
        cmd.extend(["-N", str(self.timeout)])

        # Add K_g key if provided
        if self.kg_key:
            cmd.extend(["-k", self.kg_key])

        # Add privilege level
        cmd.extend(["-L", self.privilege_level])

        return cmd

    def send_raw_command(self, raw_data: List[int]) -> Dict[str, Any]:
        """Send a raw IPMI command.

        Args:
            raw_data: List of bytes for the raw command

        Returns:
            Dict containing response data and completion code
        """
        if not self.host:
            return {'completion_code': 0xC0, 'data': [], 'error': 'Not connected'}

        try:
            cmd = self._build_ipmitool_base_cmd()
            cmd.append("raw")
            cmd.extend([str(b) for b in raw_data])

            Logger.debug(f"Executing IPMI command: {' '.join(cmd)}")

            # Create temporary file for password
            with tempfile.NamedTemporaryFile(mode='w', delete=False) as temp:
                temp_path = temp.name
                temp.write(self.password)

            try:
                # Replace password with file path in command
                password_index = cmd.index("-P") + 1
                cmd[password_index] = temp_path

                # Add password file flag
                cmd[password_index - 1] = "-f"

                # Execute command
                result = subprocess.run(
                    cmd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True,
                    timeout=self.timeout + 5
                )

            finally:
                # Remove temporary password file
                try:
                    os.unlink(temp_path)
                except:
                    pass

            if result.returncode != 0:
                error_msg = result.stderr.strip()
                Logger.error(f"IPMI command failed: {error_msg}")
                return {'completion_code': 0xC0, 'data': [], 'error': error_msg}

            # Parse output
            output = result.stdout.strip()
            if not output:
                return {'completion_code': 0, 'data': []}

            # Convert hex output to bytes
            data_bytes = []
            for part in output.split():
                try:
                    data_bytes.append(int(part, 16))
                except ValueError:
                    pass

            return {'completion_code': 0, 'data': data_bytes}

        except subprocess.TimeoutExpired:
            Logger.error(f"IPMI command timed out after {self.timeout + 5} seconds")
            return {'completion_code': 0xC3, 'data': [], 'error': 'Timeout'}
        except Exception as e:
            Logger.error(f"IPMI command error: {str(e)}")
            return {'completion_code': 0xC0, 'data': [], 'error': str(e)}

    def send_command(self, netfn: int, cmd: int, data: bytes = b'') -> Dict[str, Any]:
        """Send an IPMI command.

        Args:
            netfn: Network function code
            cmd: Command code
            data: Command data bytes

        Returns:
            Dict containing response data and completion code
        """
        raw_data = [netfn, cmd]

        # Add data bytes if provided
        if isinstance(data, bytes) and data:
            raw_data.extend(list(data))
        elif isinstance(data, list):
            raw_data.extend(data)

        return self.send_raw_command(raw_data)

    def execute_ipmitool(self, subcommand: str, args: List[str] = None) -> Dict[str, Any]:
        """Execute an ipmitool subcommand.

        Args:
            subcommand: Ipmitool subcommand
            args: Additional arguments

        Returns:
            Dict containing command result
        """
        if not self.host:
            return {'success': False, 'output': '', 'error': 'Not connected'}

        try:
            cmd = self._build_ipmitool_base_cmd()
            cmd.append(subcommand)

            if args:
                cmd.extend(args)

            Logger.debug(f"Executing ipmitool command: {' '.join(cmd)}")

            # Create temporary file for password
            with tempfile.NamedTemporaryFile(mode='w', delete=False) as temp:
                temp_path = temp.name
                temp.write(self.password)

            try:
                # Replace password with file path in command
                password_index = cmd.index("-P") + 1
                cmd[password_index] = temp_path

                # Add password file flag
                cmd[password_index - 1] = "-f"

                # Execute command
                result = subprocess.run(
                    cmd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True,
                    timeout=self.timeout + 5
                )

            finally:
                # Remove temporary password file
                try:
                    os.unlink(temp_path)
                except:
                    pass

            if result.returncode != 0:
                error_msg = result.stderr.strip()
                Logger.error(f"ipmitool command failed: {error_msg}")
                return {'success': False, 'output': '', 'error': error_msg}

            return {'success': True, 'output': result.stdout.strip(), 'error': ''}

        except subprocess.TimeoutExpired:
            Logger.error(f"ipmitool command timed out after {self.timeout + 5} seconds")
            return {'success': False, 'output': '', 'error': 'Timeout'}
        except Exception as e:
            Logger.error(f"ipmitool command error: {str(e)}")
            return {'success': False, 'output': '', 'error': str(e)}

    def parse_ipmitool_output(self, output: str, pattern: str) -> List[Dict[str, str]]:
        """Parse output from ipmitool commands.

        Args:
            output: Command output string
            pattern: Regex pattern for parsing

        Returns:
            List of dictionaries with parsed values
        """
        results = []

        try:
            # Split output into lines and process each line
            for line in output.strip().split('\n'):
                match = re.search(pattern, line)
                if match:
                    results.append(match.groupdict())
        except Exception as e:
            Logger.error(f"Error parsing ipmitool output: {str(e)}")

        return results
